#!/usr/bin/perl

=head1 NAME

ExchangeSyncDirs.pl - Get a list a directories where Exchange Server
databanks are stored so they can be transferred to the vault via Secure
Data Sync.

=head1 SYNPOSIS

ExchangeSyncDirs.pl <outputFilename>

 <outputFilename> will have the list of databank directories

=head1 DESCRIPTION

This script writes the list of directories to an output file (specified as
an input parameter) as a single line of comma-separated pathnames, e.g.,

	/path/to/workspace1,/path/to/workspace2,/path/to/workspace3

This script is called internally by securesync and is not intended for
general use.

=head1 RETURN VALUE

ExchangeSyncDirs.pl can return the following exit codes:
	
=over 5

=item *

0 - Processing was successful

=item *

1 - Transfer to the vault is currently disabled

=item *

2 - Error connecting to database

=item *

3 - Could not open output file for writing

=item *

4 - Error executing a query

=back

Corresponding event types are defined in the bp.eventtype table
with eventtypeid ranging from 2000 to 2004.

=cut

BEGIN {
}

use strict;
use Pg;
use File::Basename;
use Sys::Hostname;
use EventLogging;

my $debug = 0; # set to 1 for debug output
my $msgstr;
my $prog = basename($0);
my $dpuname = hostname();
if ($debug) {
	print "program name is $prog<==\ndpuname is $dpuname<==\n";
}

if (@ARGV != 1) {
   die "Usage: ExchangeSyncDirs.pl outputFilename\n";
}

# Connect to the database
my $connStr = "host=localhost dbname=bpdb user=postgres password=";
my $dbConn = Pg::connectdb($connStr);
unless($dbConn->status == PGRES_CONNECTION_OK) {
	# Error connecting to the database
	exit 2; 
}

# Set up an event group for logging.
my ($ret, $egrp) = geteventgroup($dbConn, 't', "$prog: Creating list of databank dirs to sync");
unless ( $ret == PGRES_TUPLES_OK ) {
	exit 4; # can't call addeventlog if this fails...
}
if ($debug) {
	print "ret is $ret, egrp is $egrp\n";
}

my $checkVaultingEnabled = 'SELECT value from bp.config WHERE key = \'vault_transfer_enabled\'';
if ($debug) {
	print "checkVaultingEnabled is $checkVaultingEnabled\n";
}
my $result = $dbConn->exec($checkVaultingEnabled);
unless ( $result->resultStatus == PGRES_TUPLES_OK ) {
	($msgstr = $dbConn->errorMessage) =~ s/\s+$//;
	logevent($dbConn,'t',1,2004,$prog,"checkVaultingEnabled: $msgstr",$dpuname,$egrp);
	exit 4;
}
my $transfer_enabled = $result->getvalue(0,0); 
if ( $transfer_enabled !~ /t|y|1/i ) {
	# Transfer to the vault is currently disabled
	logevent($dbConn,'t',1,2001,$prog,'Transfer to the vault is currently disabled',$dpuname,$egrp);
	if ($debug) {
		print "transfer_enabled is $transfer_enabled (disabled).\n";
	}
	exit 1;
}

open(OUTFILE, ">", $ARGV[0]) or 
 logevent($dbConn,'t',1,2004,$prog,'Could not open output file for writing',$dpuname,$egrp)
 and exit 3;

my $workspacename;
my $dpuid;
my $isvaulted;
my $schemaname;
my $schemaQuery;
my $schema_result;
my $setSearchPath;
my $cmd_result;
my $pathQuery;
my $path_result;

my $workspaceQuery = 'SELECT workspacename,dpuid,isvaulted FROM bp.workspace';
if ($debug) {
	print "workspaceQuery is $workspaceQuery\n";
}
$result = $dbConn->exec($workspaceQuery);
unless ( $result->resultStatus == PGRES_TUPLES_OK ) {
	($msgstr = $dbConn->errorMessage) =~ s/\s+$//;
	logevent($dbConn,'t',1,2004,$prog,"workspaceQuery: $msgstr",$dpuname,$egrp);
	exit 4;
}

if ($debug) {
	print "=== Starting loop...\n";
}

my $nrows = $result->ntuples;
if ($debug) {
	print "number of rows is $nrows\n";
}
my @row;
my $path = 1;
while ( @row = $result->fetchrow ) {
	($workspacename, $dpuid, $isvaulted) = @row;
	if ($debug) {
		print "workspacename is $workspacename\ndpuid is $dpuid\nisvaulted is $isvaulted\n";
	}
	if ($isvaulted  =~ /f|n|0/i ) {
		$nrows--;
		if ($debug) {
			print "workspace is not vaulted, nrows is $nrows, continuing...\n";
		}
		next;
	}
	$schemaQuery = 'SELECT bpexch.getworkspaceschema(\'' . $workspacename; 
	$schemaQuery .= '\',\'' . $dpuid . '\')';
	if ($debug) {
		print "schemaQuery is $schemaQuery\n";
	}
	$schema_result = $dbConn->exec($schemaQuery);
	unless ( $schema_result->resultStatus == PGRES_TUPLES_OK ) {
		($msgstr = $dbConn->errorMessage) =~ s/\s+$//;
		logevent($dbConn,'t',1,2004,$prog,"schemaQuery: $msgstr",$dpuname,$egrp);
		exit 4;
	}
	$schemaname = $schema_result->getvalue(0,0);
	if ($debug) {
		print "schemaname is $schemaname\n";
	}
	$setSearchPath = 'SET search_path TO "' . $schemaname . '"';
	if ($debug) {
		print "setSearchPath is $setSearchPath\n";
	}
	$cmd_result = $dbConn->exec($setSearchPath);
	unless ( $cmd_result->resultStatus == PGRES_COMMAND_OK ) {
		($msgstr = $dbConn->errorMessage) =~ s/\s+$//;
		logevent($dbConn,'t',1,2004,$prog,"setSearchPath: $msgstr",$dpuname,$egrp);
		exit 4;
	}
	$pathQuery = 'SELECT settingsvalue FROM settingsview WHERE settingskey = \'ServerPath\'';
	if ($debug) {
		print "pathQuery is $pathQuery\n";
	}
	$path_result = $dbConn->exec($pathQuery);
	unless ( $path_result->resultStatus == PGRES_TUPLES_OK ) {
		($msgstr = $dbConn->errorMessage) =~ s/\s+$//;
		logevent($dbConn,'t',1,2004,$prog,"pathQuery: $msgstr",$dpuname,$egrp);
		exit 4;
	}
	if ( $path_result->ntuples == 0 ) {
		$nrows--;
		if ($debug) {
			print "pathQuery returned zero tuples, nrows is $nrows, continuing...\n";
		}
		next;
	}
	if ($debug) {
		print "PATH $path: ";
		print $path_result->getvalue(0,0) . '/' . $schemaname;
		print "\n";
	}
	print OUTFILE $path_result->getvalue(0,0) . '/' . $schemaname;
	if ( $path < $nrows ) {
		print OUTFILE ",";
	}
	$path++;
}

$path--;
if ($debug) {
	print "END...$path pathnames written to $ARGV[0]\n";
}

close OUTFILE;
logevent($dbConn,'t',3,2000,$prog,"$path databank locations identified for vault synchronization.",$dpuname,$egrp);
exit 0;

