##!/bin/sh
[ -n "$BASH_VERSION" ] && enable -n echo # Forces System V echo (/bin/echo)
##
##	Script to report capacity-based license info for clients.
##	Tracked in Bugzilla 1519.
##
## Bug	Inititals	Description
## ---	---------	-----------
## 1670	ME		Did not consider multiple shares for Exchange.
## 1671	ME		Display Bare Metal consistently to screen
## 1672	ME		Put _space_ before MB in display
## 1675	ME		Need usage statement

. /etc/default/bp.ini 2> /dev/null
MAILCMD=mail
BPBIN=$BPDIR/bin
TMPNODES=/tmp/nodes$$
BPUTIL=$BPBIN/bputil
USERDATA=$BPDIR/reports.dir/user_data.rpt

# allow for verbose option to show all clients backup sizes
# 1675
if [ $# -gt 0 ]
then
	if [ "$1" = "-v" ]
	then
		VerboseFlag="y"
	else
		echo "Usage: $0 [-v]"
		exit 1
	fi
else
	VerboseFlag="n"
fi
	

clear
cat << EOF

This utility will report the current capacity configuration based
on the number of clients registered to this DPU and their associated
current occupied backup space.

EOF

#
# report_number_clients
#
# This routine uses printdb reports to calculate the number of clients
# registered to this DPU.
#
report_number_clients()
{
	# Run the priming printdb command
	$BPBIN/printdb nodes 2> /dev/null

	grep "^NODE_NAME=" $BPDIR/reports.dir/nodes.txt | awk -F"=" '{print $2}' > $TMPNODES
	set `wc -l $TMPNODES`
	NumberOfNodes=$1
	echo "Number of registered clients:	$NumberOfNodes"
}

#
# report_masters_bmtls_and_sqls
#
# This routine uses prorpt reports to calculate the total size of all clients
# last master backups.
# The report generated by prorpt is in the format:
# client last_bmtl bkup# last_mast bkup# last_incr bkup# last_sel bkup#
#
report_masters_bmtls_and_sqls()
{
	# Run the priming prorpt command
	$BPBIN/bptool -D 2> /dev/null
	TotalMasters=0
	TotalBareMetals=0
	TotalSQLs=0
	ClientNumber=1
	while [ $ClientNumber -le $NumberOfNodes ]
	do
		ClientName=`$BPUTIL -g Clients Item${ClientNumber} xxx $USERDATA`
		ClientMasterSize=`$BPUTIL -g Last_Masters ${ClientName} xxx $USERDATA`
		ClientBMSize=`$BPUTIL -g Last_BareMetals ${ClientName} xxx $USERDATA`
		ClientSQLSize=`$BPUTIL -g Last_Sqls ${ClientName} xxx $USERDATA`
		if [ "$ClientMasterSize" = "xxx" ]
		then
			if [ $VerboseFlag = "y" ]
			then
				echo "Client has no master backup:		$ClientName"
			fi
		else
			if [ $VerboseFlag = "y" ]
			then
				# 1672
				echo "Client $ClientName master backup size:		${ClientMasterSize} MB"
			fi
			TotalMasters=`expr $TotalMasters + $ClientMasterSize`
		fi
		if [ "$ClientBMSize" = "xxx" ]
		then
			if [ $VerboseFlag = "y" ]
			then
				# 1671
				echo "Client has no BareMetal backup:	$ClientName"
			fi
		else
			if [ $VerboseFlag = "y" ]
			then
				# 1671, 1672
				echo "Client $ClientName BareMetal backup size:		${ClientBMSize} MB"
			fi
			TotalBareMetals=`expr $TotalBareMetals + $ClientBMSize`
		fi
		if [ "$ClientSQLSize" = "xxx" ]
		then
			if [ $VerboseFlag = "y" ]
			then
				echo "Client has no SQL backup:		$ClientName"
			fi
		else
			if [ $VerboseFlag = "y" ]
			then
				# 1672
				echo "Client $ClientName SQL backup size:			${ClientSQLSize} MB"
			fi
			TotalSQLs=`expr $TotalSQLs + $ClientSQLSize`
		fi
		ClientNumber=`expr $ClientNumber + 1`
	done
	# 1672
	echo "Total Master Backups size:	${TotalMasters} MB"
	echo "Total BareMetal Backups size:	${TotalBareMetals} MB"
	echo "Total SQL Backups size:		${TotalSQLs} MB"
}

#
# report_exchange_masters
#
# This routine locates the DPU's samba share and locates and sizes the
# last exchange master backups.
#
report_exchange_masters()
{
	SMB_CONF=/etc/samba/smb.conf
	SMBSHARE_PATTERN='^[[:space:]]*path[[:space:]]*=[[:space:]]*'
	MAGIC_FILE="UniExch.magic"
	# Bug Number 1670 - Need to total Exchange size for all shares
	TotalExchMasters=0

	
	for sharepath in `grep -i $SMBSHARE_PATTERN $SMB_CONF | awk -F = '{ print $2 }' `
	do
		if [ ! -d $sharepath ]
		then
			if [ "$VerboseFlag" = "y" ]
			then
				echo "Network share path $SambaPath not found."
			fi
		else
			# 1670
			TotalShareMasters=0
			for fname in `find $sharepath -name $MAGIC_FILE`
			do

				clientdir=`dirname $fname`
				client=`basename $clientdir`
				if [ -d $clientdir ]
				then
					LastExchMaster=`$BPUTIL -g Unitrends Last_Master xxx ${clientdir}/UniExch.magic`
					if [ "$LastExchMaster" != "xxx" ]
					then
						LastExchMastSize=`du -sk $clientdir/$LastExchMaster | awk '{print $1}'`
						LastExchMastSize=`expr $LastExchMastSize / 1024`
						# 1670
						TotalShareMasters=`expr $TotalSharehMasters + $LastExchMastSize`
						if [ "$VerboseFlag" = "y" ]
						then
							# 1672
							echo "Client $client exchange backup size:	${LastExchMastSize} MB"
						fi
					else
						if [ "$VerboseFlag" = "y" ]
						then
							echo "No Exchange Master found:		$client"
						fi
					fi
				fi
			done
			if [ "$VerboseFlag" = "y" ]
			then
				# 1672
				echo "Total Exchange Backups size for ${sharepath}:	${TotalExchMasters} MB"
			fi
			TotalExchMasters=`expr $TotalExchMasters + $TotalShareMasters`
		fi
	done
	# 1672
	echo "Total Exchange Backups size:	${TotalExchMasters} MB"
}

cleanup()
{
	rm -f $TMPNODES
}

# 
# Main section
#

report_number_clients
report_masters_bmtls_and_sqls
report_exchange_masters
cleanup
exit 0
