#!/usr/bin/perl

use strict;
use warnings;

use Fcntl ':mode';

# This script exists because the legacy (.spr) archive catalogs and the paths
# in which the archives are stored is a mishmash of unintelligible metadata on
# the archive media.  Basically, it's a mess that is better parsed by something
# made to parse arbitrary report data rather than c.

# Output format: newline-delimited lists of comma-delimited fields:
# 0: client
# 1: type: Master|Differential|Incremental|BareMetal
#            |SQL <Full|Diff|Trans>|Exchange <Full|Diff>|CEP
# 2: relative archive path (i.e. clientname/backup_xxx)
# 3: timestamp
# 4: size, in bytes
# 5: compressed (y/n)
# 6: encrypted (y/n)
# 7: instance number (only for Exchange)
# 8: legacy sql instance (only for SQL Server)
# 9: legacy sql database (only for SQL Server)
#10: original pathname (only for localdir)

$\ = "\n";
$, = ' ';
my $bputil = "/usr/bp/bin/bputil";

my ($path, $tape);
if ($#ARGV < 1
 or ($ARGV[1] ne "0" and $ARGV[1] ne "1")) {
	my $name = $0;
	$name =~ s/.*\/([^\/]*)/$1/ if $name =~ /\//;
	print "Usage: ".$name." <archive path> <is_tape?0|1>";
	exit 1;
}
($path, $tape) = (shift, shift);

my $is_tape = 0;
if ($is_tape eq '1') {
	print "Tape not yet supported"; exit 1;
	$is_tape = 1;
}

# Don't die() on error; print to stdout and don't print the script line
if (!$path) {print "Error: an archive path must be supplied"; exit 1; }
if (! -x $path) {print "Error: cannot open archive path"; exit 1; }

my $profpath = "$path/archive.spr";
if (! -f "$profpath") {print "Error: no profile found"; exit 1; }
my $method = `$bputil -g Archive BackupMethod "" $profpath`;
chomp $method;
if (!length($method)) {print "Error: invalid archive profile"; exit 1; }

# Load the profile into memory
if (! open(FILE, "$profpath")) {
	print "Error: cannot open profile $profpath"; exit 1;
}
my @profile;
push @profile, <FILE>;
close FILE;
@profile = map { chomp; $_ } @profile;

&getBackups();

print STDERR "Finished processing archive at $path";
exit 0;


# Print the list of backups existing in the archive at $path
sub getBackups()
{
	# Load the user data report from the state, if it exists
	my @userdatarpt;
	if (opendir DIR, $path) {
		my @arr = grep /.*_state$/, readdir(DIR);
		if (my $statedir = pop @arr) {
			print STDERR "Found state directory: $path/$statedir";
			my $fname = "$path/$statedir/user_data.rpt";
			if (! -e $fname) { $fname = "$path/$statedir/dpu_clients.rpt"; }
			open(FILE, $fname)
			 and push @userdatarpt, <FILE>
			 and close FILE
			 and print STDERR "Loaded user data report";
			@userdatarpt = map { chomp; $_ } @userdatarpt;
		}
		close DIR;
	}

	my @clients = map { (/\s*Client[0-9]+=([^\s]+)/) } @profile;

	print STDERR "Processing $method archive";
	print STDERR "Found clients: @clients";

	# Date range has only the file-level types and Legacy SQL ???
	# Last backups has file-level, legacy SQL, Exchange, ...
	foreach my $client (@clients) {
		my @clientcat;
		# TODO tape seek
		next if (! -x "$path/$client");
		print STDERR "Found backup path for client $client";

		# Read clientfiles.cat to get the data we need
		# TODO tape seek; probably need to get what we can from the archive profile, or
		# extract these first...
		open(FILE, "$path/$client/clientfiles.cat")
		 and push @clientcat, <FILE>
		 and close FILE;
		@clientcat = map { chomp; $_ } @clientcat;

		foreach my $catline (@clientcat) {
			my ($type, $file, $time, $size, $comp, $encr);          # Required attributes
			my ($inst, $sqlinst, $sqldb, $origdir) = ('','','',''); # Optional attributes

			next if $catline =~ /^\s*$/;
			print STDERR "Reading line: $catline";
			my @fields = split ',', $catline;
			if ($#fields < 3) { print STDERR "Invalid line"; next; }

			$type = $fields[1];

			if ($type =~ /Master|Differential|Incremental|BareMetal/) {
				$file = "$client/".($fields[2] =~ /.*(backup_.*)/)[0];
				$time = $fields[3];
				$encr = ((defined $fields[4] and substr($fields[4],0,1) eq 'E') ? 'y' : '');
			}
			elsif ($type =~ /SQL/) {
				if ($#fields < 7) {
					# Date range archives didn't store the type, so check the user data report
					# from the state directory and see if we can get it from there.
					if ($#userdatarpt > 0) {
						my $qfname = quotemeta $fields[2];
						my @arr = grep { /,$qfname,/ } @userdatarpt;
						if ($catline = pop @arr) {
							$catline =~ s/\s*Item[0-9+]=/$client,/;
							print STDERR "New line: $catline";
							@fields = split ',', $catline;
						}
					}
					if ($#fields < 7) { print STDERR "Invalid line"; next; }
				}
				$type = "SQL".$fields[5];
				$time = $fields[6];
				($sqlinst, $sqldb) = ($fields[3], $fields[4]);
				# The path is different for date range and last backups. Why? Who knows.
				if ($method eq 'Date') {
					$file = "$client/SQL/".($fields[2] =~ /.*(backup_.*)/)[0];
				} else {
					$file = "$client/$sqlinst/$sqldb/".($fields[2] =~ /.*(backup_.*)/)[0];
				}
				$encr = ((defined $fields[8] and substr($fields[8],0,1) eq 'E') ? 'y' : '');
			}
			my @stat = stat("$path/$file");
			if (!@stat) { print STDERR "File does not exist"; next; }
			$size = $stat[7];
			$comp = (($is_tape or $file =~ /\.gz/ or `file $path/$file` =~ /gzip/) ? 'y' : 'n');
			$encr = ($encr ne '' ? $encr : is_encrypted("$path/$file", $is_tape));

			my $str = "$client|$type|$file|$time|$size|$comp|$encr|$inst|$sqlinst|$sqldb|$origdir";
			print STDERR "OUTPUT: $str";
			print $str;
		}


		# Look for Exchange types in the profile; they're not in clientfiles.cat
		my $section = '';
		my @exchlines;
		for (my $i = 0; $i <= $#profile; $i++) {
			my $line = $profile[$i];
			next if $line =~ /^\s*$/;
			if ($line =~ /^\[(.*)\]/) {
				$section = $1;
				next;
			}
			my $qc = quotemeta $client;
			if ($section =~ /ExchFull|ExchDiff/ and $line =~ /\s*Item[0-9]+=$qc,([^\s]+)/) {
				my ($type, $file, $time, $size, $comp, $encr);          # Required attributes
				my ($inst, $sqlinst, $sqldb, $origdir) = ('','','',''); # Optional attributes

				print STDERR "Exchange type $section: $client,$1";
				my @fields = split ',', $1;
				if ($#fields < 3) { print STDERR "Invalid line"; next; }

				$type = $section;
				$time = $fields[1];
				$inst = $fields[2];
				$file = "$client/appInstance/$inst/".($fields[0] =~ /.*(backup_.*)/)[0];

				my @stat = stat("$path/$file");
				if (!@stat) { print STDERR "File does not exist"; next; }
				$size = $stat[7];
				$comp = (($is_tape or $file =~ /\.gz/ or `file $path/$file` =~ /gzip/) ? 'y' : 'n');
				$encr = is_encrypted("$path/$file", $is_tape); # Missing field in Exchange

				# We interrogate the state db dump to find the instance name
				$inst = getInstNameFromState($inst);

				my $str = "$client|$type|$file|$time|$size|$comp|$encr|$inst|$sqlinst|$sqldb|$origdir";
				print STDERR "OUTPUT: $str";
				print $str;
			}
		}
	}


	# Look for Local Directories types in the profile; they're not in clientfiles.cat
	my $section = '';
	my @ldlines;
	for (my $i = 0; $i <= $#profile; $i++) {
		my $line = $profile[$i];
		next if $line =~ /^\s*$/;
		if ($line =~ /^\[(.*)\]/) {
			$section = $1;
			next;
		}
		if ($section =~ /Local_Dirs/ and $line =~ /\s*LocalDir[0-9]+=([^\s]+)/) {
			my ($type, $file, $time, $size, $comp, $encr);          # Required attributes
			my ($inst, $sqlinst, $sqldb, $origdir) = ('','','',''); # Optional attributes

			$origdir = $1;
			$origdir =~ s/\/$//;
			print STDERR "Local directory: $origdir";
			$type = 'LocalDir';
			# The file is LocalDir because the remainder of path is appended to it during restore
			$file = 'LocalDir';

			my @stat = stat("$path/$file");
			if (!@stat or !S_ISDIR($stat[2])) { print STDERR "Directory does not exist"; next; }
			$time = $stat[10];
			my @szfields = split(' ', `du -sb $path/$file`, 2);
			$size = $szfields[0];
			$comp = 'n';
			$encr = 'n';

			my $str = "DPU|$type|$file|$time|$size|$comp|$encr|$inst|$sqlinst|$sqldb|$origdir";
			print STDERR "OUTPUT: $str";
			print $str;
		}
	}
}

sub is_encrypted {
	my ($file, $is_tape) = (shift, shift);
	if ($is_tape) {
		print STDERR "Assuming backup is unencrypted to avoid seeks on tape";
		return 'n';
	}
	my $linkflag = `zcat -qf $file 2>/dev/null | dd bs=1 skip=156 count=1 2>/dev/null`;
	print STDERR "Linkflag for $file is $linkflag";
	return ($linkflag eq 'E' ? 'y' : 'n');
}

sub getInstNameFromState() {
	my $id = shift;
	my $name = "Instance ID $id";
	my $dumppath;
	my @dirs = `ls -d $path/*_state | head -1`;
	if ($#dirs != 0) {
		print STDERR "Cannot find state path"; return $name;
	}
	chomp $dirs[0];
	$dumppath=$dirs[0]."/dbdump.sql";
	if (! open(FILE, "$dumppath")) {
		print STDERR "Cannot open state db dump $dumppath"; return $name;
	}

	# Read through the dump until we get to the application_instances table
	my $section = '';
	my @ldlines;
	my $found = 0;
	while (<FILE>) {
		chomp;
		if (!$found && m/^COPY application_instances /) {
			$found = 1; next;
		}
		next if !$found;
		if (m/^$id\s+/) {
			# This is the one, return the 5th field
			my @fields = split ' ';
			close FILE;
			print STDERR "Found instance name ${fields[4]} for id $id";
			return $fields[4];
		}
		if (m/^\\\./ or m/^\s*$/) {
			print STDERR "Instance $id not found in dump";
			close FILE;
			return $name;
		}
	}
	print STDERR "Instance $id not found in dump (EOF)";
	close FILE;
	return $name;
}


