SET search_path TO public;

CREATE OR REPLACE FUNCTION table_exists(sch_name VARCHAR,tbl_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
-- table names are case insensitive, but where VARCHAR compare function is
	RETURN ((SELECT COUNT(*) FROM pg_catalog.pg_tables WHERE
	lower(schemaname)=lower(sch_name) AND lower(tablename)=lower(tbl_name))>0);
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

-- Returns TRUE if the specified table column exists
CREATE OR REPLACE FUNCTION column_exists(sch_name VARCHAR,tbl_name VARCHAR,col_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN

        RETURN((
        SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n, pg_catalog.pg_attribute a
        WHERE a.attnum > 0
                AND NOT a.attisdropped
                AND a.attrelid = c.oid
                AND c.relnamespace = n.oid
                AND lower(n.nspname) = lower(sch_name)
                AND lower(c.relname) = lower(tbl_name)
                AND lower(a.attname) = lower(col_name)
        ) > 0);

END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION sequence_exists(sch_name VARCHAR,seq_name VARCHAR)
RETURNS BOOLEAN AS
$BODY$BEGIN
    RETURN ((
	SELECT COUNT(*) FROM pg_catalog.pg_class c, pg_catalog.pg_namespace n
    WHERE c.relkind = 'S'
    	AND c.relnamespace = n.oid
		AND lower(n.nspname) = lower(sch_name)
		AND lower(c.relname) = lower(seq_name)
	) > 0);
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION create_schemas()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
	sch VARCHAR[];
BEGIN
	res := 0;
-- assign more array elements here to add schemas
	sch[1] := 'bp';
	sch[2] := 'statussync';

	FOR i IN 1..array_upper(sch,1) LOOP
		IF (SELECT COUNT(*) FROM pg_catalog.pg_namespace WHERE nspname=sch[i])=0
		THEN
			RAISE NOTICE 'Creating % schema', sch[i];
			EXECUTE 'CREATE SCHEMA ' || quote_ident(sch[i]);
		END IF;
	END LOOP;

	RETURN res;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT create_schemas();
DROP FUNCTION create_schemas();

SET search_path TO bp,public;

CREATE OR REPLACE FUNCTION create_bp()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
	mods BOOLEAN;
BEGIN
	res:=0;

-- begin core tables
IF NOT table_exists('bp', 'customers') THEN
	CREATE TABLE customers (
		customer_id SERIAL PRIMARY KEY,
		name VARCHAR(128) NOT NULL);
END IF;
	
IF NOT table_exists('bp', 'locations') THEN
	CREATE TABLE locations (
		location_id SERIAL PRIMARY KEY,
		customer_id INTEGER REFERENCES customers (customer_id) ON DELETE CASCADE,
		name VARCHAR(128) NOT NULL,
		latitude NUMERIC,
		longitude NUMERIC);
END IF;
	
IF NOT table_exists('bp', 'storage') THEN
        CREATE TABLE storage (
                storage_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
                storage_name VARCHAR(32) UNIQUE,
                storage_type VARCHAR,
                storage_usage VARCHAR,
		stateless_storage_type VARCHAR DEFAULT NULL,
		mb_size BIGINT DEFAULT 0,
		mb_free BIGINT DEFAULT 0,
		mb_used BIGINT DEFAULT 0,
		is_deleted BOOLEAN DEFAULT FALSE,
		is_purging BOOLEAN DEFAULT FALSE,
		mb_to_purge BIGINT DEFAULT 0,
		is_default BOOLEAN DEFAULT FALSE,
                is_online BOOLEAN);
        ALTER SEQUENCE storage_storage_id_seq RESTART WITH 2;
        INSERT INTO storage (storage_id, storage_name, storage_type, storage_usage, is_default, is_online)
          VALUES (1, 'Internal', 'internal', 'backup', true, true);
END IF;

IF NOT table_exists('bp', 'devices') THEN
	CREATE TABLE devices (
		device_no INTEGER PRIMARY KEY,
		dev_name VARCHAR(32) UNIQUE,
		device_type INTEGER,
		node_no INTEGER,
		capacity BIGINT,
		blocksize INTEGER,
		cur_fileset INTEGER,
		comment VARCHAR(80),
		status BIGINT,
		cur_k_position BIGINT,
		time_stamp BIGINT,
		scsi_blksize INTEGER,
		is_scsi BOOLEAN,
		dev_rw_name VARCHAR(32),
		dev_nr_name VARCHAR(32),
		dev_cmd_name VARCHAR(32),
		pid BIGINT,
		same_as_devices VARCHAR(32),
		auto_switch VARCHAR(32),
		tape_class CHAR(8),
		reservation_pid BIGINT,
		scsi_caps BIGINT,
		scsi_id INTEGER,
		scsi_lun INTEGER,
		scsi_ha INTEGER,
		tested BOOLEAN,
		drive_level INTEGER,
		fd INTEGER,
		tape_no BIGINT,
		properties BIGINT,
		juke_drive_no INTEGER,
		no_label BOOLEAN,
		mux_maxcnt INTEGER,
		mux_sched INTEGER,
		mux_running INTEGER,
		is_online BOOLEAN,
		slot_no INTEGER,
		dev_changer_name VARCHAR(30),
		use_juke BOOLEAN,
		juke_libno INTEGER,
		comment_short VARCHAR(44),
		total_write_usages BIGINT,
		total_write_time BIGINT,
		backup_no BIGINT,
		juke_slot_info INTEGER,
		juke_node_no INTEGER,
		is_deleted BOOLEAN NOT NULL DEFAULT FALSE,
		dedup BOOLEAN NOT NULL DEFAULT TRUE,
	 	storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id));
END IF;

IF NOT table_exists('bp', 'systems') THEN
	CREATE TABLE systems (
		system_id SERIAL PRIMARY KEY,
		location_id INTEGER REFERENCES locations (location_id) ON DELETE SET NULL DEFAULT NULL,
		name VARCHAR(128) NOT NULL,
		is_local_system BOOLEAN NOT NULL DEFAULT FALSE,
		host TEXT,
		version VARCHAR NOT NULL DEFAULT 'Unknown',
		role VARCHAR NOT NULL,
		is_active BOOLEAN NOT NULL DEFAULT TRUE,
		check_timestamp BIGINT NOT NULL DEFAULT 0,
	 	storage_id INTEGER NOT NULL DEFAULT 1 REFERENCES storage(storage_id),
		identity VARCHAR UNIQUE DEFAULT NULL,
		device_no INTEGER DEFAULT NULL REFERENCES devices (device_no),
		is_replication_suspended BOOLEAN DEFAULT NULL,
		suspend_toggled BIGINT DEFAULT 0,
		CONSTRAINT systems_unique_host UNIQUE (host, is_local_system));
END IF;

IF NOT table_exists('bp', 'client_credentials') THEN
	CREATE TABLE client_credentials (
			credential_id SERIAL PRIMARY KEY NOT NULL,
			username VARCHAR NOT NULL,
			password VARCHAR NOT NULL,
			domain VARCHAR,
			display_name VARCHAR,
			is_default BOOLEAN DEFAULT FALSE);
	CREATE FUNCTION ensure_one_default_credential() RETURNS TRIGGER AS
	$creds$
	DECLARE default_exists BOOLEAN := FALSE;
	BEGIN
		IF NEW.is_default = FALSE THEN
			RETURN NEW;
		END IF;
		IF TG_OP = 'UPDATE' THEN
			IF OLD.is_default = TRUE THEN
				RETURN NEW;
			END IF;
		END IF;
		SELECT INTO default_exists TRUE
			FROM bp.client_credentials WHERE is_default = TRUE LIMIT 1;
		IF (default_exists = TRUE) THEN
			RAISE EXCEPTION 'Only one credential can be the default.';
			RETURN NULL;
		ELSE
			RETURN NEW;
		END IF;
	END
	$creds$ LANGUAGE 'plpgsql';
	CREATE TRIGGER client_credentials_one_default
		BEFORE INSERT OR UPDATE on client_credentials
		FOR EACH ROW EXECUTE PROCEDURE ensure_one_default_credential();
		
	-- app_aware in application_instances depends on credential_id
        CREATE FUNCTION client_credentials_turn_off_app_aware() RETURNS TRIGGER AS $client_creds_app_aware_tag$
        BEGIN
        	-- If credential_id is set to NULL, all referring app_aware instances are false
        	UPDATE bp.application_instances SET app_aware = FALSE
        	WHERE bp.application_instances.credential_id = OLD.credential_id;
        	RETURN OLD;
        END
        $client_creds_app_aware_tag$ LANGUAGE 'plpgsql';

        CREATE TRIGGER client_credentials_turn_off_app_aware BEFORE DELETE ON client_credentials
        	FOR EACH ROW EXECUTE PROCEDURE client_credentials_turn_off_app_aware();

END IF;
		
IF NOT table_exists('bp', 'nodes') THEN
	CREATE TABLE nodes (
		node_no INTEGER PRIMARY KEY,
		node_name VARCHAR(32) UNIQUE,
		type INTEGER,
		os INTEGER,
		vendor BIGINT,
		status BIGINT,
		last_master BIGINT,
		last_master_start_time BIGINT,
		last_incremental BIGINT,
		last_incremental_start_time BIGINT,
		last_selective BIGINT,
		last_selective_start_time BIGINT,
		last_baremetal BIGINT,
		last_baremetal_start_time BIGINT,
		priority INTEGER,
		options BIGINT,
		features BIGINT,
		fstatus BIGINT,
		properties BIGINT,
		bm_media_test_date BIGINT,
		last_update BIGINT DEFAULT 0,
		credential_id INTEGER DEFAULT NULL REFERENCES client_credentials (credential_id) ON DELETE SET NULL,
		gcname VARCHAR, -- grandclient 
		generic_property INTEGER DEFAULT 0, 
		system_id INTEGER REFERENCES systems(system_id));
END IF;
		
IF NOT table_exists('bp', 'application_lookup') THEN
	-- This table identifies applications that
	--   o use the same client-DPU communications protocol
	--   o are presented to the customer as the same 'application type'
	--
	-- This table maps app_id to human readable
	--   o type of application (e.g. SQL Server)
	--   o specific application (e.g. SQL Server 2006, postgres
	--
	-- To add new applications:
	--   See application_instances for guidelines to add
	-- 
	-- app_id - applications that share the same app_id,
	--          share the same client-DPU protocol choices
	--          and so nominally execute the same DPU code
	--          and code branches.
	--          Any run-time DPU algorithm selection
	--          dependent on app_id must be independent
	--          of run-time data in the current message,
	--          and as feasible, should be independent
	--          of run-time data in prior messages.
	CREATE TABLE application_lookup (
		app_id INTEGER PRIMARY KEY,
		name VARCHAR NOT NULL,
		type VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'node_applications') THEN
	-- This table identifies applications that
	--   o were used for or are available for backup
	--   o ave available via agent on specified client (node_no)
	--
	-- app_id see application_lookup
	--
	-- current - true available for current backups
	--           false application is not availabe for current bakups,
	--                 though may become available in future,
	--                 so may be needed for restore
	--   o type of application (e.g. SQL Server)
	--   o specific application (e.g. SQL Server 2006, postgres
	--
	-- what applications are available on given node
	-- what nodes make given application available
	--
	-- node_no - identifies agent though which to communication
	--
	CREATE TABLE node_applications (
		node_no INTEGER NOT NULL REFERENCES nodes(node_no) ON DELETE CASCADE,
		app_id INTEGER NOT NULL REFERENCES application_lookup(app_id),
		current BOOLEAN NOT NULL DEFAULT TRUE,
		visible BOOLEAN NOT NULL DEFAULT TRUE,
		credential_id INTEGER DEFAULT NULL REFERENCES client_credentials (credential_id) ON DELETE SET NULL,
		PRIMARY KEY (node_no, app_id));

	CREATE FUNCTION node_applications_invalidate_children() RETURNS TRIGGER AS $node_app_tag$
	BEGIN
		-- If current is set to false, all referring applications are no longer current
		IF NEW.current = FALSE THEN
			UPDATE bp.application_instances SET current = FALSE
			WHERE current != FALSE AND node_no = NEW.node_no AND app_id = NEW.app_id;
		END IF;
		RETURN NULL;
	END
	$node_app_tag$ LANGUAGE 'plpgsql';

	CREATE TRIGGER node_applications_update_current AFTER UPDATE ON node_applications
		FOR EACH ROW EXECUTE PROCEDURE node_applications_invalidate_children();
END IF;

IF NOT table_exists('bp', 'rae_tasks') THEN
	-- This table identifies RAE tasks
	--
	-- This table maps task_id to
	--   o task XML
	--
	CREATE TABLE rae_tasks (
		task_id SERIAL PRIMARY KEY NOT NULL,
		task_xml VARCHAR
		);
END IF;

IF NOT table_exists('bp', 'application_instances') THEN
	-- This table identifies all objects that
	--   o can be backed up
	--   o are bundled and unbundled by peculiar applications
	--     (i.e. not handled as files with ctar)
	--
	-- This table maps instance_id to
	--   o node_no+app_id[+key1[+key2[+key3[+key4]]]]
	--     unique object that can be backed up
        --     or has been backed up       
	--   o app_id. (see application_lookup)
	--   o node_no
	--     client with SQL agent for communication
	--     (application and data may actually reside elsewhere,
	--      but all communication from DPU is nominally through agent)
	--
	-- Note that once set for an instance_id, keys cannot be changed.
	--
	-- The meaning of each key is tabulated here,
        --   selected by application_lookup.type
        --   (or as necessary or equivalent) by app_id.
	--
	-- type        app_id keyN meaning
	-- file-level   1-  1     unused
	-- Exchange     2- 19   1 database/storage group
	-- SQL Server  21- 39   1 server
	--                      2 instance
	--                      3 database
	--
	-- To add new applications:
	--   the decision whether or not to add an app_id, key
	--   or property should maintain orthongonality
	--   (in order to minimize usage complexity)
	--   In general, add
	--     app_id to identify changes to client-DPU protocol
	--       e.g. new fields for new versions of an application
	--     key to distinguish items that can be backed up separately
	--       e.g. different products/targets of the same application
	--     prop to identify changes that may affect the same backup
	--       e.g. SQL Server recovery model
        --
	-- All application instances must refer to valid node_applications
	--   (enforced via ON DELETE CASCADE)
	--
	CREATE TABLE application_instances (
		instance_id SERIAL PRIMARY KEY NOT NULL,
		node_no INTEGER NOT NULL,
		app_id INTEGER NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		key1 VARCHAR,
		key2 VARCHAR,
		key3 VARCHAR,
		key4 VARCHAR,
		sync BOOLEAN NOT NULL DEFAULT FALSE,
		crypt BOOLEAN NOT NULL DEFAULT FALSE,
		priority INTEGER NOT NULL DEFAULT 500,
		wvf BOOLEAN NOT NULL DEFAULT FALSE,
		credential_id INTEGER DEFAULT NULL REFERENCES client_credentials (credential_id) ON DELETE SET NULL,
		app_aware BOOLEAN DEFAULT NULL,
		inline_dedup BIGINT NOT NULL DEFAULT 0,
		identity VARCHAR,
		FOREIGN KEY (node_no, app_id) REFERENCES node_applications (node_no, app_id) ON DELETE CASCADE,
		CONSTRAINT node_app_key_unique UNIQUE (node_no, app_id, key1, key2, key3, key4),
		CONSTRAINT application_instances_identity_unique UNIQUE(identity));
END IF;

IF NOT table_exists('bp', 'application_properties') THEN
	-- This table maps instance_id to
	--   o application specific data that may change
        --
	-- key selects data to
        --   o return to application with restore, query, ...
 	--   o select options related to restore, query, ...
	--
	-- The meaning and precise string of each key is tabulated here,
        --   selected by application_lookup.type
        --   (or as necessary or equivalent) by app_id.
        --
	--  type      app_id prop_key prop_value notes
	-- file-level  1-  1 unused
	-- Exchange    2- 19 unused
	-- SQL Server 21- 39 recovery_model
	--                            full       full
	--                            simple     simple
	--                   online   
	--                     TRUE (or absent) available for backup
	--                     FALSE backup may fail
	--
	-- All application properties must refer to valid application_instances
	--   (enforced via ON DELETE CASCADE)
	--
	CREATE TABLE application_properties (
		instance_id INTEGER,
		prop_key VARCHAR,
		prop_value VARCHAR,
		FOREIGN KEY (instance_id)
			REFERENCES application_instances (instance_id)
			ON DELETE CASCADE
	);
END IF;

IF NOT table_exists('bp', 'jobs') THEN
	CREATE TABLE jobs (
		job_no BIGINT PRIMARY KEY,
		batch_no BIGINT,
		log_name VARCHAR(32),
		node_no INTEGER,
		node_group INTEGER,
		start_dir_no BIGINT,
		x_command VARCHAR(512),
		queued_date BIGINT,
		time_scheduled_for BIGINT,
		start_time BIGINT,
		elapsed_time INTEGER,
		status BIGINT,
		cmd_before VARCHAR(80),
		cmd_after VARCHAR(80),
		remote_cmd_before VARCHAR(80),
		remote_cmd_after VARCHAR(80),
		tape_no BIGINT,
		fileset_no BIGINT,
		device_no INTEGER,
		attempt_cnt INTEGER,
		time_stamp BIGINT,
		pid BIGINT,
		backup_no BIGINT,
		password BIGINT,
		blocksize INTEGER,
		verify_after INTEGER,
		jdt INTEGER,
		dependant_jobno BIGINT,
		comment VARCHAR(80),
		lcs_no INTEGER,
		percent_done INTEGER,
		operator_name VARCHAR(100),
		dependant_backupno BIGINT,
		jlast_message VARCHAR(248),
		instance_id INTEGER REFERENCES bp.application_instances (instance_id) ON DELETE SET NULL,
		schedule_id INTEGER);
END IF;

IF NOT table_exists('bp', 'iscsi') THEN
        CREATE TABLE iscsi (
                storage_id INTEGER NOT NULL,
                address VARCHAR NOT NULL,
                port VARCHAR NOT NULL,
                target VARCHAR NOT NULL,
                lun VARCHAR NOT NULL,
                uname VARCHAR,
                password VARCHAR,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                PRIMARY KEY (address, port, target, lun));
END IF;

IF NOT table_exists('bp', 'fc') THEN
        CREATE TABLE fc (
                storage_id INTEGER NOT NULL,
                wwpn VARCHAR NOT NULL,
                lun VARCHAR NOT NULL,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                CONSTRAINT fc_pkey2 PRIMARY KEY (wwpn, lun));
END IF;

IF NOT table_exists('bp', 'nas') THEN
        CREATE TABLE nas (
                storage_id INTEGER NOT NULL,
                address VARCHAR NOT NULL,
                port VARCHAR NOT NULL,
                protocol VARCHAR NOT NULL,
                uname VARCHAR,
                password VARCHAR,
                share_name VARCHAR NOT NULL,
		max_size INTEGER DEFAULT 0,
		compressed INTEGER DEFAULT 0,
		fs_passwd VARCHAR,
		prefix VARCHAR,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                CONSTRAINT nas_pkey2 PRIMARY KEY (address, port, protocol, share_name));
END IF;

IF NOT table_exists('bp', 'ndmp_nas') THEN
        CREATE TABLE ndmp_nas (
        node_no INTEGER NOT NULL,
        address VARCHAR NOT NULL,
        port INTEGER NOT NULL,
        vendor VARCHAR NOT NULL,
        version VARCHAR NOT NULL,
        FOREIGN KEY (node_no) REFERENCES nodes(node_no) ON DELETE CASCADE,
        PRIMARY KEY (node_no));
END IF;

IF NOT table_exists('bp', 'internal') THEN
        CREATE TABLE internal (
                storage_id INTEGER NOT NULL,
                disk_id VARCHAR NOT NULL,
                FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
                PRIMARY KEY (disk_id));
END IF;

IF NOT table_exists('bp', 'retention_limits') THEN
        CREATE TABLE retention_limits (
                instance_id INTEGER NOT NULL,
                min_limit INTEGER NOT NULL,
                max_limit INTEGER NOT NULL,
                legal_hold INTEGER NOT NULL DEFAULT 0,
                FOREIGN KEY (instance_id) REFERENCES application_instances (instance_id) ON DELETE CASCADE,
                PRIMARY KEY (instance_id));
END IF;

IF NOT table_exists('bp', 'tapes') THEN
	CREATE TABLE tapes (
		tape_no BIGINT PRIMARY KEY,
		backup_no BIGINT,
		label VARCHAR(80),
		status BIGINT,
		media_pool INTEGER,
		magazine_no INTEGER,
		expiration_date BIGINT,
		soft_errors INTEGER,
		hard_errors INTEGER,
		total_uncompressed_data INTEGER,
		total_compressed_data INTEGER,
		total_megs BIGINT,
		location VARCHAR(80),
		capacity BIGINT,
		when_labelled BIGINT,
		usages INTEGER,
		scsi_blksize INTEGER,
		physical_fsno BIGINT,
		juke_slot INTEGER,
		juke_slot_info INTEGER,
		juke_drive_no INTEGER,
		juke_libno INTEGER,
		tape_class CHAR(8),
		sets_of_data BIGINT,
		blocksize INTEGER,
		scratch_cnt INTEGER,
		import_no BIGINT,
		no_label BOOLEAN);
END IF;
		
IF NOT table_exists('bp', 'backups') THEN
	CREATE TABLE backups (
		backup_no BIGINT PRIMARY KEY,
		attempt_cnt INTEGER,
		node_no INTEGER,
		start_time BIGINT,
		tape_no BIGINT,
		job_no BIGINT UNIQUE,
		fileset_no BIGINT,
		elapsed_time BIGINT,
		server_name VARCHAR(32),
		operator_name VARCHAR(100),
		properties BIGINT,
		total_netpaths BIGINT,
		total_megs BIGINT,
		multivolume INTEGER,
		sync_status INTEGER,
		sync_resv INTEGER,
		status BIGINT,
		start_dir_no BIGINT,
		incomplete INTEGER,
		notbacked INTEGER,
		type INTEGER,
		verify_after INTEGER,
		is_verified BOOLEAN,
		exit_code BIGINT,
		total_kilo BIGINT,
		blocksize INTEGER,
		lcs_no INTEGER,
		x_command_short VARCHAR(256),
		all_filesets_present BOOLEAN,
		pid BIGINT,
		data_bkno BIGINT);
		-- Also see backups.instance_id & certified, added in update_bpdb.sql
END IF;
		
IF NOT table_exists('bp', 'filesets') THEN
	CREATE TABLE filesets (
		fileset_no BIGINT PRIMARY KEY,
		ssn BIGINT,
		vol_no BIGINT,
		tape_no BIGINT, 
		start_time BIGINT,
		elapsed_time BIGINT,
		total_megs BIGINT,
		fileset_megs BIGINT,
		total_netpaths BIGINT,
		server_name VARCHAR(32),
		tape_drive_name VARCHAR(32),
		status BIGINT,
		properties BIGINT,
		physical_fsno BIGINT,
		v_koffset BIGINT,
		btotal BIGINT,
		tape_dataset BIGINT,
		backup_no BIGINT,
		job_no BIGINT,
		data_format CHAR(5),
		is_multiplexed BOOLEAN,
		is_encrypted BOOLEAN,
		is_compressed BOOLEAN,
		no_label BOOLEAN,
		scsi_blksize INTEGER,
		os INTEGER,
		ssn_blocksize INTEGER,
		mux_fileset_no BIGINT,
		import_no BIGINT,
		guid BIGINT,
		device_properties BIGINT);
END IF;

IF NOT table_exists('bp', 'backup_snapshots') THEN
	CREATE TABLE backup_snapshots (
		backup_no BIGINT PRIMARY KEY
			REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		snapshot_time INTEGER NOT NULL);
END IF;

IF NOT table_exists('bp', 'sql_backups') THEN
	CREATE TABLE sql_backups (
		sql_ref BIGINT PRIMARY KEY,
		node_no INTEGER,
		database_name VARCHAR(128),
		server_instance_name VARCHAR(32),
		group_file_order BIGINT,
		file_order_no BIGINT,
		sql_type INTEGER);
END IF;

IF NOT table_exists('bp', 'backup_msg') THEN
	CREATE TABLE backup_msg (
		backup_no BIGINT PRIMARY KEY,
		messages VARCHAR,
		comment VARCHAR(480));
END IF;

-- table for the replication queue
-- used to manage the replication process
IF NOT table_exists('bp','replication_queue') THEN
	CREATE TABLE replication_queue (
		backup_no BIGINT REFERENCES bp.backups (backup_no)
			ON DELETE CASCADE,
		vault TEXT NOT NULL,
		position SERIAL,
		status INTEGER,
		rep_start BIGINT);
END IF;

-- Table for inline dedup statistics
-- Populated by the inline dedup library
-- Consumed by landing zone calculations
IF NOT table_exists('bp','backup_dedup_stats') THEN
	CREATE TABLE backup_dedup_stats (
		backup_no BIGINT REFERENCES bp.backups (backup_no)
			on DELETE CASCADE,
		--Name of the operation which occured (likely inline dedup or inplace synthesis
    operation		VARCHAR(32) NOT NULL,
		--Total number of files that passed through the inline dedup library
		total_files_count BIGINT NOT NULL,
		--Total data size of all files passed through the inline dedup library
		total_file_bytes BIGINT NOT NULL,
		--Total number of files that were deduplicated (i.e. were >8KB)
		deduped_files_count BIGINT NOT NULL,
		--Total data size of files that were counted above
		deduped_files_bytes BIGINT NOT NULL,
		--Total number of bytes in all headers (effectively ULF file size)
		total_header_bytes BIGINT NOT NULL,

		--Count of new blocks identified during the backup
		new_blocks_count BIGINT NOT NULL,
		--Size of all new blocks
		new_blocks_size BIGINT NOT NULL,
		--Count of existing blocks identified
		existing_blocks_count BIGINT NOT NULL,
		--Total size of those existing blocks
		existing_blocks_size BIGINT NOT NULL,

		--Total time spent in the inline dedup library
		total_library_sec BIGINT not NULL,
		--Total time spent writing to the SIS
		writing_sis_sec BIGINT not NULL,
		--Total time writing to the deduplicated CTAR file
		writing_ctar_sec BIGINT not NULL,
		--Total time writing to the ULF file
		writing_ulf_sec BIGINT not NULL,
		--Total time spent waiting on the space manager for write access
		smgr_wait_sec BIGINT not NULL,
		--Total time quering or writing to the deduplication metadata
		metadata_sec BIGINT not NULL,
		--Total time waiting for hashing to occur
		hashing_wait_sec BIGINT not NULL,
		--Total time spent verifying data
		verify_sec BIGINT not NULL,
		--Total number of blocks verified
		verified_blocks_count BIGINT not NULL,

		--Time of insertion of this statistics entry
		stat_time TIMESTAMP DEFAULT now(),
		PRIMARY KEY(backup_no));
END IF;

-- Table to associate BareMetal/Master backups that have been re-assigned
-- from the grandclient to a direct-attached client on the target for restore.
IF NOT table_exists('bp', 'grandclient_restores') THEN
	CREATE TABLE grandclient_restores (
		directclient INTEGER REFERENCES nodes(node_no) ON DELETE CASCADE,
		grandclient INTEGER REFERENCES nodes(node_no) ON DELETE CASCADE,
		backup_no BIGINT REFERENCES backups(backup_no) ON DELETE CASCADE,
		assoc_time BIGINT NOT NULL,
		PRIMARY KEY (directclient, grandclient, backup_no)
		);
END IF;

IF NOT table_exists('bp', 'operation_names') THEN
  -- names of operations that support progress tracking
  --
  -- Operations for which support exists to track progress are
  --   herein identified by name.
  -- Many instances of each operation may be simultaneously in progress.
  CREATE TABLE operation_names (
    name_id SERIAL PRIMARY KEY NOT NULL UNIQUE, -- faster in join than CHAR()
    name_value VARCHAR(32) NOT NULL UNIQUE);    -- feature identifier
    -- additional columns (e.g. peculiar table names)
    --   may be added to simplify coordinating C/PHP code.
	-- vaulting
  --   operation_progress.current_size - cursor position in input
  --   operation_progress.final_size   - length of input
  INSERT INTO operation_names(name_value) VALUES ('delta');
  INSERT INTO operation_names(name_value) VALUES ('signature');
  INSERT INTO operation_names(name_value) VALUES ('patch');
  INSERT INTO operation_names(name_value) VALUES ('copy');
  INSERT INTO operation_names(name_value) VALUES ('migrate');
  --   replication names:
  INSERT INTO operation_names(name_value) VALUES ('prepare');
  INSERT INTO operation_names(name_value) VALUES ('replicate');
  INSERT INTO operation_names(name_value) VALUES ('wait');
  INSERT INTO operation_names(name_value) VALUES ('rebuild');
  INSERT INTO operation_names(name_value) VALUES ('no_activity');
  INSERT INTO operation_names(name_value) VALUES ('processing');
  --   fileDedup operations:
  INSERT INTO operation_names(name_value) VALUES ('hash');
  INSERT INTO operation_names(name_value) VALUES ('dedup');
  INSERT INTO operation_names(name_value) VALUES ('redup');
  INSERT INTO operation_names(name_value) VALUES ('unhash');
-- As features become newly trackable, insert new choices above
    --   and alter TABLE operation_progress if necessary.
END IF;

IF NOT table_exists('bp', 'operation_progress') THEN
  -- last status of an operation instance in progress
  --
  -- Bibliography:
  --   Backup Professional Library (bpl.so) ABP Guide Version V
  --     Operation APIs
  --       bp_get_operation_list()
  --
  -- Note:
  --   Column snapshot_time corresponds to API index current_time
  --     but was renamed to avoid namespace conflict with
  --     builtin postgres function current_time
  --
  -- Writer (run-time)
  -- To start an operation whose progress may be tracked:
  --   INSERT an SQL record with new operation string and initial data.
  -- During operation, UPDATE (explicit columns)
  --   Current_size and final_size are instantaneous values. 
  -- Upon completion, DELETE record.
  --
  -- Reader (run-time)
  -- To determine average throughput and/or estimate completion time:
  --   poll instantaneous values, keep history in memory, and smooth.
  -- Completion status must be determined by alternate means,
  --   as record is deleted upon completion.
  --
  -- To accomodate future long duration actions (e.g. database cleanup)
	--   (between releases)
	--     IF NOT column_exists('bp', 'operation_progress', 'new_field') THEN
  --       ALTER TABLE operation_progress ADD COLUMN new_field ...
	--     END IF;
  --   (with new release)
  --      Add columns here and remove from update_bpdb.sql.
  CREATE TABLE operation_progress (
    progress_id SERIAL PRIMARY KEY NOT NULL UNIQUE, -- instance identifier
    name_id INTEGER REFERENCES operation_names(name_id) NOT NULL,
          -- operation name
    start_time    BIGINT NOT NULL, -- operation began seconds since epoch
    snapshot_time BIGINT,          -- last update seconds since epoch
    final_size    BIGINT,          -- 0, units in 100%,
                                   --    e.g. completion bytecount,
    current_size  BIGINT,          -- 0, units out of final_size,
    pid_to_kill   BIGINT DEFAULT 0,
    on_behalf_of  VARCHAR(256),    -- who is watching
    source        VARCHAR(256),    -- sender client/DPU/DPV/...
    destination   VARCHAR(256),	   -- receiver client/DPU/DPV/...
         -- backup related information
    backup_no     BIGINT,          -- nominally backups(backup_no)
    backup_type   VARCHAR(32));    -- nominally (mapped) backups(type)
END IF;

IF NOT table_exists('bp', 'operation_status') THEN
-- Unlike operation_progess, these records persist so that an operation that
-- should only occur once can leave evidence that is has been done.  For
-- example, a migration may need to take place on upgrade, or the first time
-- some subsystem runs after upgrade, but should never be performed again.
-- This table can be checked to see if the operation is pending, done, etc.
  CREATE TABLE operation_status (
    operation_id INTEGER PRIMARY KEY NOT NULL UNIQUE,
    subsystem    VARCHAR NOT NULL,
    description  VARCHAR NOT NULL,
    status       VARCHAR NOT NULL
  );
END IF;

		
IF NOT table_exists('bp', 'hash_values') THEN
  -- This hash_values table is used by file deduplication to map a
  -- hash value and its attributes to a single, more managable value --
  -- its hash_id.  The database is responsible for and performs the
  -- mapping for us.  If we could do this quicker outside of the database
  -- we should.
  CREATE TABLE hash_values (
    hash_id		BIGSERIAL PRIMARY KEY,
    hash_type		VARCHAR(8) NOT NULL,
    hashed_length	BIGINT NOT NULL,
    hash_value 		BYTEA,
    sis_offset 		BIGINT DEFAULT NULL,
    compressed_length	BIGINT DEFAULT 0,
    CONSTRAINT hash_type_length_value_unique UNIQUE (hash_type, hashed_length, hash_value)
  );

  CREATE INDEX hash_values_sis_offset_index ON bp.hash_values (sis_offset);
END IF;
		
IF NOT table_exists('bp', 'backups_hash_values') THEN
  -- This backups_hash_values table associates a file hashed for the
  -- purposes of file deduplication in a backup with its hash value and
  -- hash attributes.
  CREATE TABLE backups_hash_values (
    backup_no		BIGINT REFERENCES bp.backups (backup_no)
				    ON DELETE CASCADE ON UPDATE CASCADE,
    hash_id		BIGINT REFERENCES bp.hash_values (hash_id)
				    ON DELETE RESTRICT ON UPDATE RESTRICT,
    file_offset		BIGINT NOT NULL,
    deduplicated	BIGINT DEFAULT NULL,
    hash_incomplete	BOOLEAN DEFAULT TRUE,
    CONSTRAINT backup_no_hash_id_file_offset_unique UNIQUE (backup_no, hash_id, file_offset)
  );

  CREATE INDEX backups_hash_values_hash_id_index ON bp.backups_hash_values (hash_id);
  CREATE INDEX deduplicated_backups_hash_values_index ON bp.backups_hash_values(deduplicated) WHERE deduplicated IS NULL;
END IF;
		
IF NOT table_exists('bp', 'dedup_stats') THEN
  -- This dedup_stats table records some global statistics about our
  -- deduplication operations.
  CREATE TABLE dedup_stats (
    operation		VARCHAR(8) NOT NULL,
    done_epoch		BIGINT NOT NULL,
    bytes_processed	BIGINT NOT NULL,
    elapsed_seconds	INTEGER NOT NULL
  );
END IF;

IF table_exists('bp', 'data_reduction_stats') THEN
  -- First version of the table is obsolete
  DROP TABLE bp.data_reduction_stats;
END IF;

IF NOT table_exists('bp', 'data_reduction_ratios') THEN
  -- This records the data reduction ratios for the system.
  CREATE TABLE data_reduction_ratios (
    sample_time  BIGINT PRIMARY KEY,
    dedup        DECIMAL NOT NULL,
    compr        DECIMAL NOT NULL,
    alldr        DECIMAL NOT NULL
  );
END IF;

IF NOT table_exists('bp', 'overload_stats') THEN
  CREATE TABLE overload_stats (
    operation		VARCHAR NOT NULL,
    start_time		BIGINT NOT NULL,
    end_time		BIGINT NOT NULL,
    one_min_load_x100	INTEGER NOT NULL,
    details		VARCHAR DEFAULT NULL
  );
END IF;
		
IF NOT table_exists('bp', 'lcs') THEN
	CREATE TABLE lcs (
		lcs_no INTEGER PRIMARY KEY,
		is_online BOOLEAN,
		start_time BIGINT, 
		sched_name VARCHAR(80) UNIQUE NOT NULL,
		device_no INTEGER,
		status BIGINT,
		last_start_time BIGINT,
		last_end_time BIGINT,
		total_netpaths BIGINT,
		total_megs BIGINT,
		job_no BIGINT, 
		failure_report_is_online BOOLEAN,
		failure_report_notification_type INTEGER,
		backup_summary_is_online BOOLEAN,
		backup_summary_notification_type INTEGER,
		weekly_summary_is_online BOOLEAN,
		weekly_summary_notification_type INTEGER,
		cmd_before VARCHAR (80),
		cmd_after VARCHAR (80),
		server_name VARCHAR (32),
		last_message VARCHAR (256),
		working_element INTEGER,
		is_recurring BOOLEAN);
END IF;
		
IF NOT table_exists('bp', 'lcs_client') THEN
	CREATE TABLE lcs_client (
		lcs_no INTEGER,
		node_no INTEGER,
		type INTEGER,
		freq INTEGER,
		element_freq INTEGER,
		seq_no INTEGER,
		profile_path VARCHAR(256),
		is_online BOOLEAN,
		comment VARCHAR(80),
		status BIGINT,
		last_job BIGINT,
		last_job_status BIGINT,
		last_backup BIGINT,
		last_verify_job BIGINT,
		last_verify_status BIGINT,
		last_verify_backup BIGINT,
		last_message VARCHAR(256),
		retries_cnt INTEGER,
		last_queued_time BIGINT,
		use_juke BOOLEAN,
		verify_after INTEGER,
		priority INTEGER,
		sub_type INTEGER,
		PRIMARY KEY (lcs_no, node_no, type, freq, element_freq, seq_no));
END IF;
		
IF NOT table_exists('bp', 'lcs_hist') THEN
	CREATE TABLE lcs_hist (
		lcs_no INTEGER,
		start_time BIGINT,
		node_no INTEGER,
		job_no BIGINT,
		last_verify_job BIGINT,
		backup_no BIGINT,
		last_verify_backup BIGINT,
		status BIGINT,
		verify_after INTEGER,
		type INTEGER,
		comment VARCHAR(80),
		PRIMARY KEY (lcs_no, start_time, node_no, job_no));
END IF;

IF NOT table_exists('bp', 'library') THEN
	CREATE TABLE library (
		total_netpaths BIGINT,
		location VARCHAR(80),
		dispatcher INTEGER,
		library_no INTEGER PRIMARY KEY,
		dispatch_pid BIGINT,
		comment VARCHAR(80),
		unref_netpaths BIGINT,
		purge_status BIGINT,
		f_master_version BIGINT,
		lcs_client_version BIGINT);
END IF;
		
IF NOT table_exists('bp', 'pids') THEN
	CREATE TABLE pids (
		pid BIGINT PRIMARY KEY,
		filename VARCHAR(128),
		operator_name VARCHAR(100),
		comment VARCHAR(80),
		misc1 BIGINT,
		time_stamp BIGINT);
END IF;
-- end of core tables

-- begin alertman tables
IF NOT table_exists('bp', 'alert_source') THEN
	CREATE TABLE alert_source (
		source_id SERIAL PRIMARY KEY UNIQUE,
		source_name VARCHAR(35) NOT NULL);
END IF;
		
IF NOT table_exists('bp', 'alerts') THEN
	CREATE TABLE alerts (
		alert_id SERIAL PRIMARY KEY UNIQUE,
		alert_text TEXT,
		source_id INTEGER REFERENCES alert_source(source_id),
		target TEXT,
		severity INTEGER,
		created BIGINT,
		updated BIGINT,
		closed BOOLEAN DEFAULT FALSE);
END IF;
		
IF NOT table_exists('bp', 'temperature') THEN
	CREATE TABLE temperature (
		temp_id SERIAL PRIMARY KEY UNIQUE,
		time_stamp BIGINT,
		cpu1 DECIMAL,
		cpu2 DECIMAL,
		sys DECIMAL);
END IF;
		
IF NOT table_exists('bp', 'pci_table') THEN
	CREATE TABLE pci_table (
		bus_id VARCHAR(7) PRIMARY KEY UNIQUE,
		class_id INTEGER,
		vendor_id INTEGER,
		device_id INTEGER,
		device_name TEXT);
END IF;
		
IF NOT table_exists('bp', 'alert_runs') THEN
	CREATE TABLE alert_runs (
		run_id SERIAL PRIMARY KEY UNIQUE,
		started BIGINT,
		ended BIGINT);
END IF;
		
IF NOT table_exists('bp', 'inactive_clients') THEN
	CREATE TABLE inactive_clients (
		name TEXT PRIMARY KEY UNIQUE,
		time_stamp BIGINT);
END IF;
		
IF NOT table_exists('bp', 'disk_status') THEN
	CREATE TABLE disk_status (
		status_id INTEGER PRIMARY KEY UNIQUE,
		status TEXT);
END IF;
		
IF NOT table_exists('bp', 'unit_table') THEN
	CREATE TABLE unit_table (
		controller INTEGER,
		unit INTEGER,
		type TEXT,
		status_id INTEGER REFERENCES disk_status(status_id));
END IF;
		
IF NOT table_exists('bp', 'drive_table') THEN
	CREATE TABLE drive_table (
		controller INTEGER,
		unit INTEGER,
		port INTEGER,
		status_id INTEGER REFERENCES disk_status(status_id));
END IF;

IF NOT table_exists('bp', 'raid_units') THEN
	CREATE TABLE raid_units (
		controller VARCHAR(128) NOT NULL,
		unit VARCHAR(128) PRIMARY KEY UNIQUE,
		type VARCHAR(128) NOT NULL,
		uuid VARCHAR(128) NOT NULL,
		status INTEGER REFERENCES disk_status(status_id) NOT NULL
	);
END IF;

IF NOT table_exists('bp', 'raid_drives') THEN
	CREATE TABLE raid_drives (
		controller VARCHAR(128) NOT NULL,
		device VARCHAR(128) PRIMARY KEY UNIQUE,
		status INTEGER REFERENCES disk_status(status_id) NOT NULL,
		serial VARCHAR(128) NOT NULL,
		partition INTEGER NOT NULL DEFAULT 0
	);
END IF;
-- end of alertman tables

-- begin Notification tables
IF NOT table_exists('bp', 'notification_mgmt') THEN
	CREATE TABLE notification_mgmt(
		revision INTEGER PRIMARY KEY,
		upgrade_from INTEGER NOT NULL,
		status VARCHAR NOT NULL);
	INSERT INTO notification_mgmt(revision, upgrade_from, status)
		VALUES (0, 0, 'needs migration');
END IF;

IF NOT table_exists('bp', 'categories') THEN
	CREATE TABLE categories (
		category_id INTEGER PRIMARY KEY,
		category VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'notifications') THEN
	CREATE TABLE notifications(
		notification_id INTEGER PRIMARY KEY,
    	category_id INTEGER REFERENCES categories(category_id) NOT NULL,
    	short_description VARCHAR NOT NULL,
    	primary_text VARCHAR NOT NULL,
    	severity INTEGER NOT NULL,
    	state BOOLEAN NOT NULL,
    	action VARCHAR,
    	alert BOOLEAN NOT NULL,
    	trap BOOLEAN NOT NULL,
    	email BOOLEAN NOT NULL,
		log BOOLEAN NOT NULL DEFAULT FALSE,
    	psa BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF NOT table_exists('bp', 'notify_alerts') THEN
	CREATE TABLE notify_alerts(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
		source_id INTEGER REFERENCES alert_source(source_id) NOT NULL);
END IF;

IF NOT table_exists('bp', 'trap_types') THEN
	CREATE TABLE trap_types(
		trap_type_id INTEGER PRIMARY KEY,
		trap_type VARCHAR NOT NULL);

END IF;

IF NOT table_exists('bp', 'notify_traps') THEN
	CREATE TABLE notify_traps(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
    	trap_type_id INTEGER REFERENCES trap_types(trap_type_id) NOT NULL,
		oid VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'notify_emails') THEN
	CREATE TABLE notify_emails(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
	--  The section/keyword to look up in master.ini for the recipient list
		to_section VARCHAR NOT NULL,
		to_keyword VARCHAR NOT NULL,
	--  from_name is just the from_name part of from_name@dpu.domain.com
		from_name VARCHAR NOT NULL,
		subject VARCHAR NOT NULL,
		body VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'notify_psa') THEN
	CREATE TABLE notify_psa(
		notification_id INTEGER PRIMARY KEY
			REFERENCES notifications (notification_id) ON DELETE CASCADE,
		short_description VARCHAR NOT NULL,
		long_description VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'audit_history') THEN
	CREATE TABLE audit_history (
	id SERIAL PRIMARY KEY,
	notification_id INTEGER NOT NULL,
	event_time TIMESTAMP DEFAULT now(),
	username VARCHAR NOT NULL,
	system VARCHAR NOT NULL,
	message VARCHAR NOT NULL);
END IF;
-- end of Notification tables

-- begin SDS stats tables
IF NOT table_exists('bp', 'sync_type') THEN
	CREATE TABLE sync_type (
		type_id SERIAL PRIMARY KEY NOT NULL UNIQUE,
		type_name VARCHAR(32) NOT NULL);
END IF;

IF NOT table_exists('bp', 'sds_stats') THEN
	CREATE TABLE sds_stats (
		client_name VARCHAR(32) NOT NULL,
		backup_no BIGINT NOT NULL,
		sync_type INTEGER REFERENCES sync_type(type_id) NOT NULL,
		destination VARCHAR,
		presync_seconds INTEGER,
		sync_seconds INTEGER,
		postsync_seconds INTEGER,
		done_epoch BIGINT NOT NULL,
		bytes_on_dpu BIGINT,
		bytes_syncd BIGINT,
		bytes_on_dpv BIGINT,
		message text NOT NULL,
		stat_time TIMESTAMP DEFAULT now());
END IF;

-- Records in this table block securesync from starting a new operation for
-- the given client and syncInfo.syncType while a cleanup is occurring.
IF NOT table_exists('bp', 'securesync_guard') THEN
	CREATE TABLE securesync_guard (
		client_name VARCHAR NOT NULL,
		syncinfo_type VARCHAR NOT NULL,
		guard_time TIMESTAMP DEFAULT now(),
		PRIMARY KEY (client_name, syncinfo_type));
END IF;
-- end of SDS stats tables

-- begin security tables
IF NOT table_exists('bp', 'users') THEN
	CREATE TABLE users (
		user_id SERIAL PRIMARY KEY,
		username VARCHAR(64) UNIQUE,
		password VARCHAR(64) NOT NULL,
		vault_user BOOLEAN DEFAULT FALSE);

	-- Trigger function to delete nvp values when users are deleted --
	CREATE OR REPLACE FUNCTION delete_user_nvp() RETURNS TRIGGER AS $del_nvp$
	BEGIN
		DELETE FROM bp.nvp WHERE OLD.username = nvp_name AND nvp_name != 'root';
		RETURN OLD;
	END
	$del_nvp$ LANGUAGE 'plpgsql';
 
	CREATE TRIGGER del_from_users_to_nvp
		AFTER DELETE ON bp.users FOR EACH ROW
		EXECUTE PROCEDURE delete_user_nvp();

	-- Trigger function to delete sessions when users are deleted --
	CREATE OR REPLACE FUNCTION delete_user_sessions() RETURNS TRIGGER AS $del_sessions$
	BEGIN
		DELETE FROM bp.sessions WHERE OLD.user_id = user_id;
		RETURN OLD;
	END
	$del_sessions$ LANGUAGE 'plpgsql';
 
	CREATE TRIGGER del_from_users_to_sessions
		AFTER DELETE ON bp.users FOR EACH ROW
		EXECUTE PROCEDURE delete_user_sessions();
END IF;
	
IF NOT table_exists('bp', 'sessions') THEN
	CREATE TABLE sessions (
		user_id INTEGER NOT NULL,
		uuid VARCHAR,
		expiration BIGINT NOT NULL,
		PRIMARY KEY (uuid));
END IF;

IF NOT table_exists('bp', 'priv_lookup') THEN
	CREATE TABLE priv_lookup (
		priv_level INTEGER PRIMARY KEY,
		priv_name VARCHAR(64) NOT NULL,
		priv_description TEXT);
END IF;
	
IF NOT table_exists('bp', 'superusers') THEN
	CREATE TABLE superusers (
		user_id INTEGER PRIMARY KEY REFERENCES users (user_id) ON DELETE CASCADE
	);
END IF;

IF NOT table_exists('bp', 'forum_users') THEN
	CREATE TABLE forum_users (
		user_id INTEGER PRIMARY KEY
			REFERENCES users (user_id) ON DELETE CASCADE,
		credential_id INTEGER NOT NULL
			REFERENCES client_credentials (credential_id) ON DELETE CASCADE,
		salesforce_id VARCHAR NOT NULL
	);
END IF;
	
IF NOT table_exists('bp', 'customer_priv') THEN
	CREATE TABLE customer_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		customer_id INTEGER NOT NULL REFERENCES customers (customer_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, customer_id));
END IF;
	
IF NOT table_exists('bp', 'location_priv') THEN
	CREATE TABLE location_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		location_id INTEGER NOT NULL REFERENCES locations (location_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, location_id));
END IF;
	
IF NOT table_exists('bp', 'system_priv') THEN
	CREATE TABLE system_priv (
		user_id INTEGER NOT NULL REFERENCES users (user_id) ON DELETE CASCADE,
		system_id INTEGER NOT NULL REFERENCES systems (system_id) ON DELETE CASCADE,
		priv_level INTEGER NOT NULL REFERENCES priv_lookup (priv_level) ON DELETE SET DEFAULT DEFAULT 0,
		PRIMARY KEY (user_id, system_id));
END IF;
-- end of security tables

IF NOT table_exists('bp', 'async_query_job') THEN
	CREATE TABLE async_query_job (
		real_pid INTEGER,
		timestr VARCHAR,
		fork_pid INTEGER,
		status VARCHAR,
		temptable_name VARCHAR,
		PRIMARY KEY (real_pid, timestr));
	
	-- Create the trigger function for this table (this only happens on the first
	-- time this table is added to the schema).  
	-- NOTE: if this ever needs to be modified on an upgrade, put the new version
	-- in create_functions.sql.
	CREATE OR REPLACE FUNCTION bp.DropTmpfilesearchTbl() RETURNS TRIGGER AS
	$async_tbl_tag$
	DECLARE
		cmd VARCHAR;
		tbl VARCHAR;
	BEGIN 
		SELECT INTO tbl relname FROM pg_catalog.pg_class WHERE relname = OLD.temptable_name ;
		IF tbl IS NULL
		THEN
	--		RAISE NOTICE 'table not found: %', OLD.temptable_name;
			RETURN NULL;
		ELSE
	--		RAISE NOTICE 'dropping table: %', OLD.temptable_name ;
			cmd := 'DROP TABLE ' || OLD.temptable_name ;
			EXECUTE cmd;
			RETURN OLD;
		END IF;
	END
	$async_tbl_tag$
	LANGUAGE 'plpgsql' VOLATILE;

	-- Now define the trigger
	CREATE TRIGGER drop_tmp_tbl_trig AFTER DELETE ON async_query_job FOR EACH ROW
	EXECUTE PROCEDURE DropTmpfilesearchTbl();

END IF;

IF NOT table_exists('bp', 'nvp') THEN
	CREATE TABLE nvp (
		nvp_type VARCHAR NOT NULL,
		nvp_name VARCHAR NOT NULL,
		item_name VARCHAR NOT NULL,
		item_value VARCHAR,
		PRIMARY KEY (nvp_type, nvp_name, item_name));
END IF;

IF NOT table_exists('bp', 'virtual_reservation') THEN
        CREATE TABLE virtual_reservation (
                vf_id INTEGER NOT NULL,
                vf_type INTEGER NOT NULL,
                vf_size INTEGER,
                PRIMARY KEY (vf_id, vf_type));
END IF;

IF NOT table_exists('bp', 'node_volumes') THEN
	CREATE TABLE node_volumes (
		node_no INTEGER NOT NULL REFERENCES nodes (node_no) ON DELETE CASCADE,
		volume VARCHAR NOT NULL,
		CONSTRAINT node_volumes_unique_volume UNIQUE (node_no, volume));
END IF;

-- begin scheduling tables
IF NOT table_exists('bp', 'selection_lists') THEN
	CREATE TABLE selection_lists (
		selection_list_id SERIAL PRIMARY KEY,
		name VARCHAR(40) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		type VARCHAR NOT NULL,
		family VARCHAR NOT NULL,
		node_no INTEGER,
		read_only BOOLEAN NOT NULL,
		netmounts BOOLEAN NOT NULL,
		registry BOOLEAN NOT NULL,
		tempfiles BOOLEAN NOT NULL,
		readmounts BOOLEAN NOT NULL,
		allmounts BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'selection_list_filenames') THEN
	CREATE TABLE selection_list_filenames (
		selection_list_id INTEGER NOT NULL REFERENCES selection_lists (selection_list_id) ON DELETE CASCADE,
		filename VARCHAR NOT NULL);

		CREATE INDEX selection_list_filenames_selection_list_id ON bp.selection_list_filenames(selection_list_id);
END IF;

IF NOT table_exists('bp', 'option_lists') THEN
	CREATE TABLE option_lists (
		option_list_id SERIAL PRIMARY KEY,
		name VARCHAR(40) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		type VARCHAR NOT NULL,
		family VARCHAR NOT NULL,
		read_only BOOLEAN NOT NULL,
		device_name VARCHAR REFERENCES devices (dev_name) ON DELETE SET NULL ON UPDATE CASCADE,
		read_locking INTEGER NOT NULL,
		speed_option BOOLEAN NOT NULL,
		verify_level INTEGER NOT NULL,
		depth INTEGER NOT NULL,
		comment VARCHAR,
		raw_device VARCHAR,
		raw_device_command VARCHAR,
		before_command VARCHAR(79),
		after_command VARCHAR(79),
		make_catalog BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'strategies') THEN
	CREATE TABLE strategies (
		strategy_id SERIAL PRIMARY KEY,
		name VARCHAR UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		read_only BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'calendars') THEN
	CREATE TABLE calendars (
		calendar_id SERIAL PRIMARY KEY,
		name VARCHAR(40) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		family VARCHAR NOT NULL,
		read_only BOOLEAN NOT NULL);
END IF;

IF NOT table_exists('bp', 'events') THEN
  -- repeatable points in time
  --
	CREATE TABLE events (
		event_id SERIAL PRIMARY KEY,
		calendar_id INTEGER REFERENCES calendars (calendar_id) ON DELETE CASCADE,
		strategy_id INTEGER REFERENCES strategies (strategy_id) ON DELETE CASCADE,
		summary INTEGER,
		dtstart BIGINT,      -- start of freq unit earlier than first occurance
		freq INTEGER,        -- include/bp/bpstatus.h: Logical Client Set Backup
		interval INTEGER,    -- freq units (days, weeks, ...) between repetitions
		bymonthday INTEGER,
		description VARCHAR,
		CONSTRAINT event_has_parent CHECK (calendar_id IS NOT NULL OR strategy_id IS NOT NULL),
		CONSTRAINT event_has_one_parent CHECK (calendar_id IS NULL OR strategy_id IS NULL));
END IF;

IF NOT table_exists('bp', 'byday') THEN
	CREATE TABLE byday (
		event_id INTEGER NOT NULL REFERENCES events (event_id) ON DELETE CASCADE,
		day_no INTEGER NOT NULL,
		CONSTRAINT byday_unique_day_no UNIQUE (event_id, day_no));
END IF;

IF NOT table_exists('bp', 'byhour') THEN
	CREATE TABLE byhour (
		event_id INTEGER NOT NULL REFERENCES events (event_id) ON DELETE CASCADE,
		hour_no INTEGER NOT NULL,
		CONSTRAINT byhour_unique_hour_no UNIQUE (event_id, hour_no));
END IF;

IF NOT table_exists('bp', 'byminute') THEN
	CREATE TABLE byminute (
		event_id INTEGER NOT NULL REFERENCES events (event_id) ON DELETE CASCADE,
		minute_no INTEGER NOT NULL,
		CONSTRAINT byminute_unique_minute_no UNIQUE (event_id, minute_no));
END IF;

IF NOT table_exists('bp', 'schedules') THEN
	CREATE TABLE schedules (
		schedule_id SERIAL PRIMARY KEY,
		name VARCHAR(40) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		calendar_id INTEGER REFERENCES calendars (calendar_id),
		app_id INTEGER REFERENCES application_lookup (app_id),
		association_type VARCHAR NOT NULL DEFAULT 'none',
		association_id	VARCHAR,
		option_list_id INTEGER REFERENCES option_lists (option_list_id),
		add_new_instances BOOLEAN NOT NULL DEFAULT FALSE,
		enabled BOOLEAN NOT NULL DEFAULT FALSE,	
		email_report BOOLEAN NOT NULL DEFAULT FALSE,
		failure_report BOOLEAN NOT NULL DEFAULT FALSE,
		before_command VARCHAR(79),
		after_command VARCHAR(79),
		run_only BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF column_exists('bp', 'schedules', 'association_type') THEN
CREATE OR REPLACE FUNCTION validate_schedule_association()
                RETURNS TRIGGER AS $sched_assoc_tag$
        DECLARE
                assoc_type VARCHAR;
                assoc_id VARCHAR;
                name VARCHAR;
        BEGIN
                SELECT s.name, association_type, association_id
                        FROM bp.schedules s WHERE s.schedule_id = NEW.schedule_id
                        INTO name, assoc_type, assoc_id;

                IF assoc_type = 'client' THEN
                        PERFORM TRUE FROM bp.application_instances
                                WHERE node_no != assoc_id::integer AND instance_id = NEW.instance_id;
                        IF FOUND THEN
                                RAISE EXCEPTION 'Client % cannot be in schedule %', assoc_id, name;
                                RETURN NULL;
                        END IF;
                ELSIF assoc_type = 'esx' THEN
                        PERFORM TRUE FROM bp.vmware_vms LEFT JOIN bp.vmware_vcenter_assoc USING (esx_uuid)
                                WHERE COALESCE(vcenter_uuid,esx_uuid) = assoc_id AND instance_id = NEW.instance_id;
                        IF NOT FOUND THEN
                                RAISE EXCEPTION 'Instance % cannot be in schedule %', NEW.instance_id, name;
                                RETURN NULL;
                        END IF;
                END IF;

                RETURN NEW;
        END
        $sched_assoc_tag$ LANGUAGE 'plpgsql';

	IF NOT table_exists('bp', 'schedule_client_assoc') THEN
		CREATE TABLE  bp.schedule_client_assoc(
			schedule_id INTEGER REFERENCES schedules(schedule_id) ON DELETE CASCADE,
			instance_id INTEGER REFERENCES application_instances(instance_id) ON DELETE CASCADE,
			type INTEGER,
			option_list_id INTEGER REFERENCES option_lists (option_list_id) ON DELETE CASCADE,
			selection_incl_id INTEGER REFERENCES selection_lists (selection_list_id) ON DELETE CASCADE,
			selection_excl_id INTEGER REFERENCES selection_lists (selection_list_id) ON DELETE CASCADE,
			PRIMARY KEY (schedule_id, instance_id, type));

		CREATE TRIGGER validate_schedule_association BEFORE INSERT OR UPDATE ON bp.schedule_client_assoc
			FOR EACH ROW EXECUTE PROCEDURE validate_schedule_association();
	END IF;
END IF;

IF NOT table_exists('bp', 'schedule_lcs') THEN
	CREATE TABLE schedule_lcs (
		schedule_id INTEGER NOT NULL REFERENCES schedules (schedule_id),
		lcs_no INTEGER NOT NULL REFERENCES lcs (lcs_no) DEFERRABLE INITIALLY DEFERRED);
END IF;

IF NOT table_exists('bp', 'lcs_client_assoc') THEN
	CREATE TABLE lcs_client_assoc (
		lcs_no INTEGER,
		node_no INTEGER,
		type INTEGER,
		option_list_id INTEGER REFERENCES option_lists (option_list_id),
		selection_incl_id INTEGER REFERENCES selection_lists (selection_list_id),
		selection_excl_id INTEGER REFERENCES selection_lists (selection_list_id),
		PRIMARY KEY (lcs_no, node_no, type));
END IF;

IF NOT table_exists('bp', 'schedule_vm_container_lookup') THEN
	-- This table identifies containers that
	--   o are used for regular expressions
	--
	-- This table maps container_id to human readable
	--   o container names to be sent to the hypervisor
	--   o to retrieve list of VMs per container
	--
	-- 
	-- container_id - this id will be stored in the regular
	--          expression column in the schedules table. 
	CREATE TABLE schedule_vm_container_lookup (
		container_id INTEGER PRIMARY KEY,
		hypervisor VARCHAR NOT NULL,
		name VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'schedule_vm_regex_lookup') THEN
	-- This table identifies actions that
	--   o are used for regular expressions
	--
	-- This table maps regex_action_id to human readable
	--   o action names to be used to create a regular
	--   o expression to be used to dynamically retrieve VMs
	--
	-- 
	-- regex_action_id - this id will be stored in the regular
	--          expression column in the schedules table. 
	CREATE TABLE schedule_vm_regex_lookup (
		regex_action_id INTEGER PRIMARY KEY,
		name VARCHAR NOT NULL);
END IF;

-- end scheduling tables

IF NOT table_exists('bp', 'reports') THEN
	CREATE TABLE reports (
		report_id SERIAL PRIMARY KEY,
		name VARCHAR(32) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		based_on VARCHAR,
		fields VARCHAR(65536));
END IF;

IF NOT table_exists('bp', 'bpcleanup_stats') THEN
	CREATE TABLE bpcleanup_stats (
		node_no INTEGER PRIMARY KEY REFERENCES nodes (node_no) ON DELETE CASCADE,
		last_clean_time BIGINT DEFAULT 0,
		last_elapsed_time BIGINT DEFAULT 0,
		current_elapsed_time BIGINT DEFAULT 0,
		next_dir_no BIGINT DEFAULT 0);
END IF;

IF NOT table_exists('bp', 'synthesis_jobs') THEN
	CREATE TABLE synthesis_jobs (
		synthesis_id SERIAL PRIMARY KEY,
		instance_id INTEGER NOT NULL
			REFERENCES application_instances (instance_id) ON DELETE CASCADE,
		type INTEGER NOT NULL,
		start_time BIGINT NOT NULL,
		device_no INTEGER NOT NULL
			REFERENCES devices (device_no) ON DELETE CASCADE,
		in_progress BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF NOT table_exists('bp', 'synthesis_failures') THEN
	CREATE TABLE synthesis_failures (
		backup_no INTEGER NOT NULL
			REFERENCES backups (backup_no) ON DELETE CASCADE,
		failure_time BIGINT NOT NULL,
		reason VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'managers') THEN
	CREATE TABLE managers (
		ip VARCHAR PRIMARY KEY,
		hostname VARCHAR,
		version VARCHAR DEFAULT 'Unknown');
END IF; 
IF NOT table_exists('bp', 'snmp_config') THEN
	CREATE TABLE snmp_config (
	id SERIAL PRIMARY KEY,
	destination VARCHAR UNIQUE NOT NULL,
	community VARCHAR DEFAULT 'public',
	enabled BOOLEAN);
END IF;

IF NOT table_exists('bp', 'snmp_history') THEN
	CREATE TABLE snmp_history (
	id SERIAL PRIMARY KEY,
	oid VARCHAR NOT NULL,
	time_sent TIMESTAMP DEFAULT now(),
	destination VARCHAR NOT NULL,
	community VARCHAR,
	traptype VARCHAR,
	object VARCHAR,
	description VARCHAR,
	severity INT,
	status INT);
END IF;

IF NOT table_exists('bp', 'psa_tools') THEN
	CREATE TABLE psa_tools (
	psa_tool_id INTEGER PRIMARY KEY,
	psa_tool_name VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'psa_config') THEN
	CREATE TABLE psa_config (
	psa_id SERIAL PRIMARY KEY,
	psa_tool_id INTEGER NOT NULL UNIQUE REFERENCES psa_tools(psa_tool_id)
		ON DELETE RESTRICT,
	url VARCHAR NOT NULL,
	company_id VARCHAR NOT NULL,
	credential_id INTEGER NOT NULL REFERENCES client_credentials(credential_id)
		ON DELETE RESTRICT,
	is_default BOOLEAN DEFAULT false);

	CREATE FUNCTION ensure_one_default_psa() RETURNS TRIGGER AS
	$psa$
	DECLARE default_exists BOOLEAN := FALSE;
	BEGIN
		IF NEW.is_default = FALSE THEN
			RETURN NEW;
		END IF;
		IF TG_OP = 'UPDATE' THEN
			IF OLD.is_default = TRUE THEN
				RETURN NEW;
			END IF;
		END IF;
		SELECT INTO default_exists TRUE
			FROM bp.psa_config WHERE is_default = TRUE LIMIT 1;
		IF (default_exists = TRUE) THEN
			RAISE EXCEPTION 'Only one PSA application can be the default.';
			RETURN NULL;
		ELSE
			RETURN NEW;
		END IF;
	END
	$psa$ LANGUAGE 'plpgsql';
	CREATE TRIGGER psa_config_one_default
		BEFORE INSERT OR UPDATE on psa_config
		FOR EACH ROW EXECUTE PROCEDURE ensure_one_default_psa();
END IF;

IF NOT table_exists('bp', 'psa_history') THEN
	CREATE TABLE psa_history (
	id SERIAL PRIMARY KEY,
	notification_id INTEGER NOT NULL,
	time_sent TIMESTAMP DEFAULT now(),
	severity INT NOT NULL,
	tool_name VARCHAR NOT NULL,
	ticket_id VARCHAR NOT NULL,
	description VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'schedule_history') THEN
	CREATE TABLE schedule_history (
		schedule_id INTEGER,
		instance_id INTEGER, --0 reserved for "no events to launch" status
		scheduled_start_time BIGINT,
		status_time BIGINT,
		type INTEGER,    --LCST_* backup type
		job_no INTEGER,
		backup_no INTEGER,
		queued INTEGER,
		status_string VARCHAR,
		PRIMARY KEY (schedule_id, instance_id, scheduled_start_time));
END IF;


IF NOT table_exists('bp', 'state') THEN
	-- Unique identifier of database state, used to guarantee
	-- that node_no and/or instance_id values match
	CREATE TABLE bp.state (
		install_id VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'dirs') THEN
	-- Part of the new "main" schema, here to avoid migration later
	CREATE TABLE bp.dirs (
		dir_no SERIAL PRIMARY KEY,
		dname VARCHAR UNIQUE NOT NULL);
	CREATE INDEX dirs_dname_lcase ON bp.dirs(lower(dname));
END IF;

IF NOT table_exists('bp', 'dirs_non_ctar') THEN
	CREATE TABLE bp.dirs_non_ctar (
		dir_no SERIAL PRIMARY KEY,
		dname VARCHAR UNIQUE NOT NULL);
	INSERT INTO bp.dirs_non_ctar (dname) VALUES ('');
	INSERT INTO bp.dirs_non_ctar (dname) VALUES ('@@@:/');
	CREATE INDEX dirs_non_ctar_dname_lcase ON bp.dirs_non_ctar(lower(dname));
END IF;

IF NOT table_exists('bp', 'backup_files') THEN
	CREATE TABLE bp.backup_files (
		backup_no INTEGER REFERENCES bp.backups(backup_no) ON DELETE CASCADE,
		end_backup_no INTEGER NOT NULL DEFAULT 0,
		dir_no INTEGER NOT NULL REFERENCES bp.dirs(dir_no),
		fname VARCHAR NOT NULL,
		type INT2 NOT NULL,
		size BIGINT NOT NULL,
		mod_date INTEGER NOT NULL,
		blk_offset BIGINT NOT NULL,
		blocks BIGINT NOT NULL,
		PRIMARY KEY (backup_no, blk_offset));
	-- This non-intuitive column order was the fastest indexing strategy
	-- for queries in 6.0 synthesis, QuickSeek, file browse, and updatedb.
	CREATE INDEX backup_files_backup_dir_idx ON bp.backup_files(dir_no, backup_no, end_backup_no);
	CREATE INDEX backup_files_backup_no_index ON bp.backup_files (backup_no, end_backup_no);
END IF;

IF NOT table_exists('bp', 'backup_files_non_ctar') THEN
	CREATE TABLE bp.backup_files_non_ctar (
		backup_no INTEGER REFERENCES bp.backups(backup_no) ON DELETE CASCADE,
		end_backup_no INTEGER NOT NULL DEFAULT 0,
		dir_no INTEGER NOT NULL REFERENCES bp.dirs_non_ctar(dir_no),
		fname VARCHAR NOT NULL,
		type INT2 NOT NULL,
		size BIGINT NOT NULL,
		mod_date INTEGER NOT NULL,
		blk_offset BIGINT NOT NULL,
		blocks BIGINT NOT NULL,
		PRIMARY KEY (backup_no, dir_no, fname));
	-- NDMP has no synthesis operations, so there seems little point in
	-- mimicking the column order from backup_files.
	CREATE INDEX backup_files_non_ctar_backup_dir_idx ON bp.backup_files_non_ctar (backup_no, end_backup_no, dir_no);
	CREATE INDEX backup_files_non_ctar_backup_no_index ON bp.backup_files_non_ctar (backup_no, end_backup_no);
END IF;

IF NOT table_exists('bp', 'archive_process_ctrl') THEN
	CREATE TABLE bp.archive_process_ctrl(
		set_id INTEGER NOT NULL,
		child_pid INTEGER NOT NULL,
		backup_no INTEGER NOT NULL,
		progress_mb BIGINT NOT NULL,
		process_timestamp INTEGER NOT NULL,
		status INTEGER NOT NULL,
		PRIMARY KEY (set_id, child_pid));
END IF;

IF NOT table_exists('bp', 'archive_process_log') THEN
	CREATE TABLE bp.archive_process_log(
		set_id INTEGER NOT NULL,
		pid INTEGER NOT NULL,
		event_time INTEGER NOT NULL,
		log_msg VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'archive_backup_pool') THEN
	CREATE TABLE bp.archive_backup_pool(
		set_id INTEGER NOT NULL,
		archive_index INTEGER NOT NULL,
		backup_no INTEGER NOT NULL,
		size_mb INTEGER NOT NULL,
		assigned_pid INTEGER NOT NULL DEFAULT 0,
		PRIMARY KEY (set_id, backup_no));
END IF;

IF NOT table_exists('bp', 'archive_gc_info') THEN
	CREATE TABLE bp.archive_gc_info(
		archive_instance_id INTEGER UNIQUE NOT NULL,
		gcname VARCHAR,
		system_name VARCHAR,
		system_uuid VARCHAR,
		app_uuid VARCHAR,
		PRIMARY KEY (archive_instance_id));
END IF;

IF NOT table_exists('bp', 'archive_instances') THEN
	-- This table allows us to properly enforce integrity with NULL keyN values
	-- while providing a snapshot of the client at the time of backup
	CREATE TABLE bp.archive_instances (
		archive_instance_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		instance_id INTEGER UNIQUE
			REFERENCES bp.application_instances(instance_id),
		client_name VARCHAR NOT NULL,
		app_id INTEGER NOT NULL,
		key1 VARCHAR,
		key2 VARCHAR,
		key3 VARCHAR,
		key4 VARCHAR,
		client_os INTEGER NOT NULL,
		CONSTRAINT instance_client_app_key_unique UNIQUE (instance_id, client_name, app_id, key1, key2, key3, key4));
END IF;

-- Used in trigger to disallow modification to referenced archive profiles
CREATE OR REPLACE FUNCTION bp.archive_profile_no_modification()
	RETURNS TRIGGER AS $app_sched_tag$
DECLARE
	referenced BOOLEAN := FALSE;
BEGIN
	IF TG_OP = 'INSERT' THEN
		SELECT INTO referenced TRUE
		  FROM bp.archive_sets WHERE profile_id = NEW.profile_id LIMIT 1;
	ELSIF TG_OP = 'UPDATE' OR TG_OP = 'DELETE' THEN
		SELECT INTO referenced TRUE
		  FROM bp.archive_sets WHERE profile_id = OLD.profile_id LIMIT 1;
	END IF;
	IF (referenced = TRUE) THEN
		RAISE EXCEPTION 'Archive profile cannot be modified: it is referenced in one or more archive sets.';
		RETURN NULL;
	ELSE
		RETURN NEW;
	END IF;
END
$app_sched_tag$ LANGUAGE 'plpgsql';

IF NOT table_exists('bp', 'archive_profiles') THEN
	CREATE TABLE bp.archive_profiles (
		profile_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		description VARCHAR(128) NOT NULL,
		target VARCHAR NOT NULL,
		target_slots VARCHAR NOT NULL DEFAULT '',
		range_end INTEGER NOT NULL,
		range_size INTEGER NOT NULL,
		install_id VARCHAR NOT NULL,
		revision INTEGER NOT NULL,
		append BOOLEAN NOT NULL DEFAULT FALSE,
		compress BOOLEAN NOT NULL DEFAULT FALSE,
		encrypt BOOLEAN NOT NULL DEFAULT FALSE,
		dedup BOOLEAN NOT NULL DEFAULT FALSE,
		fastseed BOOLEAN NOT NULL DEFAULT FALSE,
		email_report BOOLEAN NOT NULL DEFAULT FALSE,
		retention_days INTEGER NOT NULL DEFAULT 0,
		purge BOOLEAN NOT NULL DEFAULT FALSE);

		CREATE TRIGGER archive_profiles_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profiles
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_slots') THEN
	CREATE TABLE archive_profile_slots (
		profile_id INTEGER REFERENCES archive_profiles (profile_id),
		slot INTEGER,
		PRIMARY KEY (profile_id, slot));
END IF;

IF NOT table_exists('bp', 'archive_profile_backtypes') THEN
	CREATE TABLE bp.archive_profile_backtypes (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		backtype INTEGER NOT NULL,
		PRIMARY KEY (profile_id, backtype));

		CREATE TRIGGER archive_profile_backtypes_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_backtypes
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_clients') THEN
	CREATE TABLE bp.archive_profile_clients (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		node_no INTEGER NOT NULL
			REFERENCES bp.nodes(node_no) ON DELETE RESTRICT,
		PRIMARY KEY (profile_id, node_no));

		CREATE TRIGGER archive_profile_clients_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_clients
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_instances') THEN
	CREATE TABLE bp.archive_profile_instances (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		archive_instance_id INTEGER NOT NULL
			REFERENCES bp.archive_instances(archive_instance_id) ON DELETE RESTRICT,
		PRIMARY KEY (profile_id, archive_instance_id));

		CREATE TRIGGER archive_profile_instances_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_instances
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_profile_localdirs') THEN
	CREATE TABLE bp.archive_profile_localdirs (
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE CASCADE,
		localdir VARCHAR NOT NULL,
		PRIMARY KEY (profile_id, localdir));

		CREATE TRIGGER archive_profile_localdirs_no_modification_constraint
			BEFORE INSERT OR UPDATE OR DELETE ON bp.archive_profile_localdirs
			FOR EACH ROW EXECUTE PROCEDURE bp.archive_profile_no_modification();
END IF;

IF NOT table_exists('bp', 'archive_schedule_profiles') THEN
	CREATE TABLE bp.archive_schedule_profiles (
		schedule_id INTEGER UNIQUE NOT NULL
			REFERENCES bp.schedules(schedule_id)
			ON DELETE CASCADE,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT);
END IF;

IF NOT table_exists('bp', 'archive_job_profiles') THEN
	CREATE TABLE bp.archive_job_profiles (
		job_no INTEGER PRIMARY KEY
			REFERENCES bp.jobs(job_no) ON DELETE CASCADE,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT);
END IF;

IF NOT table_exists('bp', 'archive_sets') THEN
	CREATE TABLE bp.archive_sets (
		archive_set_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		profile_id INTEGER NOT NULL
			REFERENCES bp.archive_profiles(profile_id) ON DELETE RESTRICT,
		media_label VARCHAR NOT NULL,
		originating_asset VARCHAR NOT NULL,
		creation_timestamp INTEGER NOT NULL,
		elapsed_secs INTEGER NOT NULL,
		archive_set_path VARCHAR NOT NULL,
		state_success BOOLEAN NOT NULL,
		status VARCHAR NOT NULL,
		pid INTEGER NOT NULL);
END IF;

IF NOT table_exists('bp', 'archive_devices') THEN
	CREATE TABLE archive_devices (
		device_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		parent_device_id INTEGER REFERENCES archive_devices (device_id),
		type VARCHAR NOT NULL,
		available BOOLEAN NOT NULL,
		name VARCHAR UNIQUE NOT NULL,
		vendor VARCHAR NOT NULL,
		model VARCHAR NOT NULL,
		serial VARCHAR NOT NULL);
END IF;

IF NOT table_exists('bp', 'archive_device_settings') THEN
	CREATE TABLE archive_device_settings (
		device_id INTEGER REFERENCES archive_devices (device_id),
		name VARCHAR NOT NULL,
		value VARCHAR,
		PRIMARY KEY (device_id, name));
END IF;

IF NOT table_exists('bp', 'archive_disks') THEN
	CREATE TABLE bp.archive_disks (
		serial_no VARCHAR NOT NULL,
		archive_set_id INTEGER NOT NULL
			REFERENCES bp.archive_sets(archive_set_id) ON DELETE CASCADE,
		slot INTEGER NOT NULL,                     -- Media sequence of set
		start_mark INTEGER NOT NULL DEFAULT 0,     -- Where to seek for set
		first_archive INTEGER NOT NULL DEFAULT -1, -- Which archive 1st on media
		PRIMARY KEY (serial_no, archive_set_id),
		CONSTRAINT archive_set_slot_unique UNIQUE (archive_set_id, slot));
END IF;

IF NOT table_exists('bp', 'archive_tapes') THEN
	CREATE TABLE archive_tapes (
		serial_no VARCHAR NOT NULL, -- same as archive_disks.serial_no
		fully_known BOOLEAN NOT NULL, -- false if imported until known EOD
		last_seen_slot INTEGER NOT NULL,
		last_seen_device_db_id INTEGER
			REFERENCES bp.archive_devices ON DELETE SET NULL,
		label VARCHAR NOT NULL,
		barcode VARCHAR NOT NULL,
		init_count INTEGER NOT NULL,
		init_date INTEGER NOT NULL,
		first_init_date INTEGER NOT NULL,
		is_hw_compressed BOOLEAN NOT NULL,
		is_hw_encrypted BOOLEAN NOT NULL,
		is_worm BOOLEAN NOT NULL,
                cap_status INTEGER,
                mb_written INTEGER,
                is_prepared INTEGER NOT NULL DEFAULT 0,
		PRIMARY KEY (serial_no));
END IF;

IF NOT table_exists('bp', 'archives') THEN
	CREATE TABLE bp.archives (
		archive_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		archive_set_id INTEGER NOT NULL
			REFERENCES bp.archive_sets(archive_set_id) ON DELETE RESTRICT,
		orig_description VARCHAR,
		orig_backup_no INTEGER,
		orig_backup_prop BIGINT NOT NULL DEFAULT 0,
		dep_backup_no INTEGER,
		archive_instance_id INTEGER
			REFERENCES bp.archive_instances(archive_instance_id)
			ON DELETE RESTRICT,
		-- Properties of the backup
		backtype INTEGER NOT NULL,
		orig_time INTEGER NOT NULL,
		orig_mib_size INTEGER NOT NULL,
		file_mib_size INTEGER NOT NULL,
		file_count INTEGER NOT NULL DEFAULT 0,
		compressed BOOLEAN NOT NULL,
		encrypted BOOLEAN NOT NULL,
		deduped BOOLEAN NOT NULL DEFAULT FALSE,
		vmware_template BOOLEAN NOT NULL DEFAULT FALSE,
		xen_template BOOLEAN NOT NULL DEFAULT FALSE,
		-- Properties of the archive
		elapsed_secs INTEGER NOT NULL,
		archive_path VARCHAR,
		archive_success BOOLEAN NOT NULL,
		error_string VARCHAR,
		CONSTRAINT archive_set_orig_desc_backup_no_unique UNIQUE (archive_set_id, orig_description, orig_backup_no),
		CONSTRAINT archive_has_source
			CHECK (orig_description IS NOT NULL OR orig_backup_no IS NOT NULL));

	CREATE OR REPLACE FUNCTION bp.archives_instance_in_archive_set()
		RETURNS TRIGGER AS $app_sched_tag$
	DECLARE
		exists BOOLEAN := FALSE;
	BEGIN
		IF (NEW.archive_instance_id IS NULL AND NEW.orig_backup_no IS NULL) THEN
			RETURN NEW;
		END IF;
		IF (NEW.backtype = 1038) THEN
			RETURN NEW;
		END IF;

		-- Check instance assignments
		SELECT INTO exists TRUE
		  FROM bp.archive_profile_instances api
		  JOIN bp.archive_sets USING (profile_id)
		  WHERE archive_sets.archive_set_id = NEW.archive_set_id
		    AND api.archive_instance_id = NEW.archive_instance_id
		  LIMIT 1;

		IF (exists = TRUE) THEN
			RETURN NEW;
		END IF;

		-- Check client assignments
		SELECT INTO exists TRUE
		  FROM bp.archive_profile_clients
		  JOIN bp.archive_sets USING (profile_id)
		  JOIN bp.nodes USING (node_no)
		  JOIN bp.archive_instances ai ON (nodes.node_name = ai.client_name)
		  WHERE ai.instance_id IS NOT NULL
		    AND archive_sets.archive_set_id = NEW.archive_set_id
		    AND ai.archive_instance_id = NEW.archive_instance_id
		  LIMIT 1;

		IF (exists = TRUE) THEN
			RETURN NEW;
		ELSE
			RAISE EXCEPTION 'Archive instance % must be in archive set %',
				NEW.archive_instance_id, NEW.archive_set_id;
			RETURN NULL;
		END IF;
	END
	$app_sched_tag$ LANGUAGE 'plpgsql';

	CREATE TRIGGER archives_instance_in_archive_set_constraint_constraint
		BEFORE INSERT OR UPDATE ON bp.archives
		FOR EACH ROW EXECUTE PROCEDURE bp.archives_instance_in_archive_set();
END IF;

-- After 6.2, archive_files will not be used anymore, child tables of it will be created during archive.
IF NOT table_exists('bp', 'archive_files') THEN
	CREATE TABLE bp.archive_files (
		archive_id INTEGER NOT NULL
			REFERENCES bp.archives(archive_id) ON DELETE CASCADE,
		dir_no INTEGER NOT NULL REFERENCES bp.dirs(dir_no),
		fname VARCHAR NOT NULL,
		type INT2 NOT NULL,
		size BIGINT NOT NULL,
		mod_date INTEGER NOT NULL,
		blocks BIGINT NOT NULL,
		blk_offset BIGINT NOT NULL,
		PRIMARY KEY (archive_id, blk_offset));
	CREATE INDEX archive_files_archive_dir_idx
			ON bp.archive_files(archive_id, dir_no);
END IF;

IF NOT table_exists('bp', 'archive_restore_specs') THEN
	CREATE TABLE bp.archive_restore_specs (
		job_no INTEGER PRIMARY KEY REFERENCES bp.jobs(job_no) ON DELETE CASCADE,
		archive_id INTEGER NOT NULL
			REFERENCES bp.archives(archive_id) ON DELETE CASCADE,
		device_no INTEGER NOT NULL
			REFERENCES bp.devices(device_no) ON DELETE CASCADE,
		node_no INTEGER REFERENCES bp.nodes(node_no) ON DELETE CASCADE,
		instance_id INTEGER
			REFERENCES bp.application_instances(instance_id) ON DELETE CASCADE,
		dr_mode BOOLEAN NOT NULL DEFAULT FALSE,
		gc_mode BOOLEAN NOT NULL DEFAULT FALSE);
END IF;

IF NOT table_exists('bp', 'archive_restore_files') THEN
	CREATE TABLE bp.archive_restore_files (
		job_no INTEGER NOT NULL
			REFERENCES bp.archive_restore_specs(job_no) ON DELETE CASCADE,
		pathname VARCHAR NOT NULL,
		is_inclusion BOOLEAN NOT NULL,
		PRIMARY KEY (job_no, pathname));
END IF;

IF NOT table_exists('bp', 'archive_backups') THEN
	CREATE TABLE bp.archive_backups (
		job_no BIGINT NOT NULL
			REFERENCES bp.jobs(job_no) ON DELETE CASCADE,
		backup_no BIGINT NOT NULL,
		PRIMARY KEY (job_no, backup_no));
END IF;

IF NOT table_exists('bp', 'vmware_esx_servers') THEN
	CREATE TABLE bp.vmware_esx_servers (
		esx_uuid VARCHAR NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		version VARCHAR NOT NULL,
		num_cpu_sockets INTEGER NOT NULL DEFAULT 1,
		PRIMARY KEY(esx_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_vcenter_servers') THEN
	CREATE TABLE bp.vmware_vcenter_servers (
		vcenter_uuid VARCHAR NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		PRIMARY KEY (vcenter_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_vcenter_assoc') THEN
	CREATE TABLE bp.vmware_vcenter_assoc (
		esx_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_esx_servers ON DELETE CASCADE,
		vcenter_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_vcenter_servers ON DELETE CASCADE,
		PRIMARY KEY (esx_uuid, vcenter_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_connections') THEN
	CREATE TABLE bp.vmware_connections (
		connection_id SERIAL NOT NULL,
		esx_uuid VARCHAR
			REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		vcenter_uuid VARCHAR
			REFERENCES bp.vmware_vcenter_servers (vcenter_uuid) ON DELETE CASCADE,
		login VARCHAR NOT NULL,
		password VARCHAR NOT NULL,
		address VARCHAR NOT NULL,
		PRIMARY KEY(connection_id));
END IF;

IF NOT table_exists('bp', 'vmware_hardware_hashes') THEN
	CREATE TABLE bp.vmware_hardware_hashes (
		backup_no BIGINT NOT NULL
			REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		hardware_hash VARCHAR NOT NULL,
		PRIMARY KEY (backup_no));
END IF;


IF NOT table_exists('bp', 'vmware_snapshots') THEN
	CREATE TABLE bp.vmware_snapshots (
		backup_no BIGINT NOT NULL
			REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		change_id VARCHAR NOT NULL,
		PRIMARY KEY (backup_no));
END IF;

IF NOT table_exists('bp', 'vmware_vms') THEN
	CREATE TABLE bp.vmware_vms (
		vm_uuid VARCHAR NOT NULL,
		esx_uuid VARCHAR NOT NULL
			REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		connection_id INTEGER NOT NULL
			REFERENCES bp.vmware_connections (connection_id) ON DELETE RESTRICT,
		instance_id INTEGER
			REFERENCES bp.application_instances (instance_id) NOT NULL,
		name VARCHAR NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		change_tracking_enabled BOOLEAN NOT NULL,
		virtual_hw_version VARCHAR NOT NULL,
		parent_type INTEGER NOT NULL DEFAULT 0,
		parent_key VARCHAR NOT NULL DEFAULT '',
		is_template BOOLEAN NOT NULL DEFAULT FALSE,
		vm_instance_uuid VARCHAR UNIQUE,
		CONSTRAINT vms_pkey2 UNIQUE (vm_uuid, vm_instance_uuid));
END IF;

-- VMWare Resource Pools
IF NOT table_exists('bp', 'vmware_resource_pools') THEN
	CREATE TABLE bp.vmware_resource_pools (
		esx_uuid                   VARCHAR NOT NULL REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		resource_pool_key    VARCHAR NOT NULL,
		resource_pool_name VARCHAR NOT NULL,
		parent_type               INTEGER NOT NULL,
		parent_key                VARCHAR NOT NULL,
		PRIMARY KEY (esx_uuid, resource_pool_key));
END IF;

-- VMWare vApps
IF NOT table_exists('bp', 'vmware_vapps') THEN
	CREATE TABLE bp.vmware_vapps (
		esx_uuid      VARCHAR NOT NULL REFERENCES bp.vmware_esx_servers (esx_uuid) ON DELETE CASCADE,
		vapp_uuid    VARCHAR NOT NULL,
		vapp_name  VARCHAR NOT NULL,
		parent_type INTEGER NOT NULL,
		parent_key   VARCHAR NOT NULL,
		PRIMARY KEY (esx_uuid, vapp_uuid));
END IF;

IF NOT table_exists('bp', 'vmware_vm_disks') THEN
	CREATE TABLE bp.vmware_vm_disks (
		instance_id INTEGER
			REFERENCES bp.application_instances (instance_id) ON DELETE CASCADE,
		name VARCHAR NOT NULL,
		disk_id VARCHAR NOT NULL,
		key INTEGER NOT NULL,
		mb_size INTEGER NOT NULL,
		is_excluded BOOLEAN NOT NULL DEFAULT FALSE,
		rdm INTEGER NOT NULL,
		disk_mode INTEGER NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		PRIMARY KEY (instance_id, key));
END IF;

IF NOT table_exists('bp', 'schedule_client_assoc') THEN

	CREATE OR REPLACE FUNCTION restrict_schedule_migration_phase_insert()
	       RETURNS TRIGGER AS $tag$
	       DECLARE
		cnt INTEGER;
	BEGIN
		SELECT COUNT(*) FROM bp.schedule_migration_phase INTO cnt;

		IF cnt = 0 THEN
			RETURN NEW;
		ELSIF cnt = 1 THEN
			RETURN NULL;
		END IF;

	END
	$tag$ LANGUAGE 'plpgsql';

	IF NOT table_exists('bp', 'schedule_migration_phase') THEN
		CREATE TABLE bp.schedule_migration_phase (
			completed_phase INTEGER NOT NULL DEFAULT 0,
			status VARCHAR NOT NULL DEFAULT 'incomplete',
			fscheds INTEGER NOT NULL DEFAULT 0,
			ascheds INTEGER NOT NULL DEFAULT 0,
			arc_scheds INTEGER NOT NULL DEFAULT 0);

		CREATE TRIGGER restrict_schedule_migration_phase_insert BEFORE INSERT ON bp.schedule_migration_phase
		FOR EACH ROW EXECUTE PROCEDURE restrict_schedule_migration_phase_insert();

		INSERT INTO bp.schedule_migration_phase(completed_phase, status) VALUES(0, 'incomplete');
	END IF;
END IF;

IF NOT sequence_exists('bp', 'restore_group_seq') THEN
	CREATE SEQUENCE bp.restore_group_seq;
END IF;

IF NOT sequence_exists('bp', 'nodes_node_no_seq') THEN
	CREATE SEQUENCE bp.nodes_node_no_seq;
END IF;

IF NOT sequence_exists('bp', 'nodes_gclient_seq') THEN
	CREATE SEQUENCE bp.nodes_gclient_seq;
END IF;

IF NOT sequence_exists('bp', 'filesets_fileset_no_seq') THEN
	CREATE SEQUENCE bp.filesets_fileset_no_seq;
END IF;

IF NOT sequence_exists('bp', 'tapes_tape_no_seq') THEN
	CREATE SEQUENCE bp.tapes_tape_no_seq;
END IF;

IF NOT sequence_exists('bp', 'jobs_job_no_seq') THEN
	CREATE SEQUENCE bp.jobs_job_no_seq;
END IF;

IF NOT sequence_exists('bp', 'backups_backup_no_seq') THEN
	CREATE SEQUENCE bp.backups_backup_no_seq;
END IF;

IF NOT sequence_exists('bp', 'lcs_lcs_no_seq') THEN
	CREATE SEQUENCE bp.lcs_lcs_no_seq;
END IF;




-- The snmp_temp table was an interim table in 4.1.1 for
-- implementing traps that are not in the alerts table,
-- and is no longer needed since these traps are now
-- represented in the notification framework.
IF table_exists('bp', 'snmp_temp') THEN
	DROP TABLE snmp_temp; 
END IF;

IF NOT table_exists('bp', 'virtual_clients') THEN
	CREATE TABLE bp.virtual_clients (
		virtual_id SERIAL PRIMARY KEY,
		-- A node can have no more than one virtual client
		node_no INTEGER UNIQUE NOT NULL REFERENCES nodes(node_no),
		cpus INTEGER NOT NULL,
		memory BIGINT NOT NULL,
		disks VARCHAR,
		incl_vols INTEGER REFERENCES selection_lists(selection_list_id)
			ON DELETE SET NULL,
		excl_vols INTEGER REFERENCES selection_lists(selection_list_id)
			ON DELETE SET NULL,
		-- audit_verify: periodically boot & email a screenshot
		audit_verify BOOLEAN NOT NULL DEFAULT FALSE,	
		-- After going to live mode, VM becomes invalid for virtual restore
		valid BOOLEAN NOT NULL DEFAULT TRUE,
		-- User can toggle disabled to prevent/enable virtual restores 
		disabled BOOLEAN NOT NULL DEFAULT FALSE,
		port INTEGER,
		current_state VARCHAR NOT NULL,
		pending_state VARCHAR,
		-- When the VM first went into live mode
		live_time BIGINT DEFAULT 0,
		-- Backup currently being restored
		cur_backup_no BIGINT DEFAULT 0,
		last_checkpoint VARCHAR DEFAULT NULL,
		-- Flag indicates Restore Daemon should process config updates
		rdu BOOLEAN NOT NULL DEFAULT TRUE,
		-- Support for external hypervisor as host of virtual client
		vm_name VARCHAR,
		vmware_host INTEGER REFERENCES bp.vmware_connections(connection_id)
			ON DELETE RESTRICT,
		hyperv_host INTEGER REFERENCES bp.nodes(node_no)
			ON DELETE RESTRICT,
		vm_guid VARCHAR UNIQUE,
		ip_addr VARCHAR UNIQUE);
END IF;

IF NOT table_exists('bp', 'virtual_clients_queue') THEN
	CREATE TABLE bp.virtual_clients_queue (
		virtual_id INTEGER REFERENCES bp.virtual_clients (virtual_id) ON DELETE CASCADE,
		backup_no BIGINT REFERENCES bp.backups (backup_no) ON DELETE CASCADE,
		cur_restore_job_no BIGINT REFERENCES bp.jobs (job_no) ON DELETE SET NULL,
		PRIMARY KEY (virtual_id, backup_no));
END IF;

IF NOT table_exists('bp', 'virtual_clients_appinst') THEN
	CREATE TABLE bp.virtual_clients_appinst (
		virtual_id INTEGER REFERENCES bp.virtual_clients (virtual_id) ON DELETE CASCADE,
		instance_id INTEGER REFERENCES bp.application_instances (instance_id) ON DELETE CASCADE,
		-- Most recently restored backup and its start_time
		last_backup_no BIGINT DEFAULT 0,
		last_backup_time BIGINT DEFAULT 0,
		PRIMARY KEY (virtual_id, instance_id));
END IF;

-- Begin summary tables
IF NOT table_exists('bp', 'inventory_sync') THEN
	CREATE TABLE inventory_sync (
		pid INTEGER,
		current_state INTEGER NOT NULL, -- 0=not running
		final_status INTEGER,  -- 0=success; otherwise, #errors during sync
		progress_pct INTEGER,  -- current percent progress (1-100)
		start_time BIGINT,     -- start of sync (epoch)
		update_time BIGINT,    -- last time this record was updated
		finish_time BIGINT,    -- completion time of last sync
		last_message VARCHAR
	);
	INSERT INTO bp.inventory_sync (pid, current_state, final_status,
			progress_pct, start_time, update_time, finish_time, last_message)
	VALUES(0, 0, 0, 0, 0, 0, 0, NULL);
	CREATE RULE  insert_not_allowed AS ON INSERT TO bp.inventory_sync
	DO INSTEAD NOTHING;
END IF;

IF NOT table_exists('bp', 'summary_counts') THEN
	CREATE TABLE summary_counts (
		backup_errors INTEGER,    -- number of backup errors in the last 7 days
		backup_protected INTEGER, -- instances protected by a successful backup
		backup_not_protected INTEGER,      -- #instances that are not protected
		replicated_backup_errors INTEGER,  -- #replicated backup errors, 7 days
		replicated_backup_protected INTEGER, -- #replicated instances protected
		replicated_backup_not_protected INTEGER, -- #replicated & not protected
		replication_errors INTEGER,      -- replication errors over last 7 days
		replication_protected INTEGER,    -- #instances configured to replicate
		restore_active_flr INTEGER,  -- number of currently active FLR sessions
		restore_active_ir INTEGER,    -- number of currently active IR sessions
		restore_recent INTEGER,    -- number of regular restores in last 7 days
		archive_errors INTEGER,      -- number of archiving errors, last 7 days
		archive_protected INTEGER,  -- #instances successfully archived, 7 days
		wir_errors INTEGER,       -- number of failed WIR restores, last 7 days
		wir_protected INTEGER,            -- number of clients protected by WIR
		rpo_days INTEGER,       -- #days for computing rpo_protected, default 7
		rpo_protected INTEGER,-- #instances have recovery point within rpo_days
		rpo_not_protected INTEGER, -- #instances, no recovery point in rpo_days
		replicated_rpo_protected INTEGER, --#replicated assets,RP w/in rpo_days
		replicated_rpo_not_protected INTEGER, --#replicated no RP w/in rpo_days
		last_update BIGINT -- epoch timestamp of when the metrics were computed
	);
	INSERT INTO bp.summary_counts (backup_errors, backup_protected,
		backup_not_protected, replicated_backup_errors,
		replicated_backup_protected, replicated_backup_not_protected,
		replication_errors, replication_protected, restore_active_flr,
		restore_active_ir, restore_recent, archive_errors, archive_protected,
		wir_errors, wir_protected, rpo_days, rpo_protected, rpo_not_protected,
		replicated_rpo_protected, replicated_rpo_not_protected, last_update)
			VALUES (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0);
	CREATE RULE  no_insert_summary_counts AS ON INSERT TO bp.summary_counts
	DO INSTEAD NOTHING;
END IF;

IF NOT table_exists('bp', 'summary_charts') THEN
	CREATE TABLE summary_charts (
		backup_elapsed_time BIGINT NOT NULL DEFAULT 0, --sum elapsed backup time
		backup_count INTEGER NOT NULL DEFAULT 0,   -- number of backups this day
		backup_read_size BIGINT NOT NULL DEFAULT 0,     -- sum data from clients
		backup_write_size BIGINT NOT NULL DEFAULT 0, --either compr or read size
		backup_compress_size BIGINT NOT NULL DEFAULT 0,   -- sum compressed size
		backup_read_time BIGINT NOT NULL DEFAULT 0, -- secs read on data channel
		backup_write_time BIGINT NOT NULL DEFAULT 0, -- secs writing data buffer
		backup_compress_time BIGINT NOT NULL DEFAULT 0,-- secs doing compression
		replication_read_size BIGINT NOT NULL DEFAULT 0,    -- replication reads
		replication_write_size BIGINT NOT NULL DEFAULT 0,  -- replication writes
		replication_read_time BIGINT NOT NULL DEFAULT 0,  -- replicate read time
		replication_write_time BIGINT NOT NULL DEFAULT 0, --replicate write time
		replication_start_lag BIGINT NOT NULL DEFAULT 0,  -- bu cmpl - rep start
		replication_prot_gap BIGINT NOT NULL DEFAULT 0,    -- bu cmpl - rep cmpl
		replication_successes BIGINT NOT NULL DEFAULT 0,  -- success cmpl repl's
		replication_retries BIGINT NOT NULL DEFAULT 0,    -- unsuccessful repl's
		replication_insert_time BIGINT NOT NULL DEFAULT 0, -- src repl inst time
		restore_size_total BIGINT NOT NULL DEFAULT 0,       -- sum data restored
		restore_elapsed_time BIGINT NOT NULL DEFAULT 0,  -- total time restoring
		restore_count INTEGER NOT NULL DEFAULT 0, -- number of restores this day
		archive_size_total BIGINT NOT NULL DEFAULT 0,   -- sum data amt archived
		archive_elapsed_time BIGINT NOT NULL DEFAULT 0,  -- total time archiving
		archive_count INTEGER NOT NULL DEFAULT 0, -- number of archives this day
		insert_date DATE PRIMARY KEY NOT NULL UNIQUE -- day in YYYY-MM-DD format
	);
END IF;

IF NOT table_exists('bp', 'target_summary_charts') THEN
	CREATE TABLE target_summary_charts (
		src_system_id INTEGER NOT NULL REFERENCES
			systems (system_id) ON DELETE CASCADE,  -- sys id of repl src
		replication_read_size BIGINT NOT NULL DEFAULT 0,    -- replication reads
		replication_write_size BIGINT NOT NULL DEFAULT 0,  -- replication writes
		replication_read_time BIGINT NOT NULL DEFAULT 0,  -- replicate read time
		replication_write_time BIGINT NOT NULL DEFAULT 0, --replicate write time
		replication_start_lag BIGINT NOT NULL DEFAULT 0,  -- bu cmpl - rep start
		replication_prot_gap BIGINT NOT NULL DEFAULT 0,    -- bu cmpl - rep cmpl
		replication_successes BIGINT NOT NULL DEFAULT 0,  -- success cmpl repl's
		replication_retries BIGINT NOT NULL DEFAULT 0,    -- unsuccessful repl's
		replication_insert_time BIGINT NOT NULL DEFAULT 0, -- src repl inst time
		insert_date DATE NOT NULL,                   -- day in YYYY-MM-DD format
		PRIMARY KEY (src_system_id, insert_date)
	);
END IF;
-- End summary tables

IF NOT table_exists('bp', 'storage_history') THEN
	CREATE TABLE storage_history (
		storage_id INTEGER NOT NULL,      -- storage identifier from storage table
		insert_date DATE NOT NULL,                     -- day in YYYY-MM-DD format
		mb_size BIGINT NOT NULL DEFAULT 0,                -- size of storage in MB
		mb_used BIGINT NOT NULL DEFAULT 0,         -- amount of storage used in MB
		mb_free BIGINT NOT NULL DEFAULT 0,         -- amount of free storage in MB
		FOREIGN KEY (storage_id) REFERENCES storage(storage_id) ON DELETE CASCADE,
		PRIMARY KEY (storage_id, insert_date)
	);
END IF;

-- begin update_history tables
IF NOT table_exists('bp', 'update_history') THEN
	CREATE TABLE bp.update_history (
		start_time INTEGER PRIMARY KEY,
		end_time INTEGER ,
		status VARCHAR(128) ,
		version_from VARCHAR(128) ,
		version_to VARCHAR(128));
END IF;

IF NOT table_exists('bp', 'update_history_errors') THEN
	CREATE TABLE bp.update_history_errors (
		start_time INTEGER REFERENCES bp.update_history(start_time),
		error VARCHAR(128) NOT NULL);
END IF;

IF NOT table_exists('bp', 'update_history_mod') THEN
	CREATE TABLE bp.update_history_mod (
		last_modification INTEGER);
END IF;
-- end of update_history tables

-- begin sync_summary tables
IF NOT table_exists('bp', 'sync_summary') THEN
	CREATE TABLE bp.sync_summary (
		system_id INTEGER NOT NULL REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		total_mb_size BIGINT DEFAULT 0,
		total_mb_free BIGINT DEFAULT 0,
		registered_assets INTEGER DEFAULT 0,
		archive VARCHAR,
		version_status VARCHAR DEFAULT 'unknown',
		PRIMARY KEY (system_id));
END IF;

-- end of sync_summary tables


-- Begin appliance agent table and functions(s)
IF NOT table_exists('bp', 'modification_time') THEN
	CREATE TABLE modification_time (
		table_name VARCHAR PRIMARY KEY UNIQUE NOT NULL,
		last_changed BIGINT,
		replicate BOOLEAN DEFAULT true);
	
		
	-- This is one nasty function because of the tables that get updated very frequently, without 
	-- actual changes.
	-- Trigger function to update modification_times table --
	CREATE OR REPLACE FUNCTION bp.update_modification_time() RETURNS TRIGGER AS
		$modification_tag$
		DECLARE name VARCHAR;
		name_row RECORD;
		new_record TEXT;
		old_record TEXT;
		different BOOLEAN := false;
		BEGIN
		-- Updates are special. Can happen to existing records, without 
		-- change in data. E.g., bp.storage.is_online used to be updated 
		-- to the same value every xx seconds by populate_alerts.
		IF (TG_OP = 'UPDATE') THEN
			IF ((TG_RELNAME = 'nodes') OR (TG_RELNAME = 'bp.nodes')) THEN
			-- Or OLD = NEW -- 
			IF (OLD.node_no = NEW.node_no AND
			OLD.node_name = NEW.node_name AND
			OLD.type = NEW.type AND
			OLD.os = NEW.os AND
			OLD.vendor = NEW.vendor AND
			OLD.status = NEW.status AND
			OLD.last_master	= NEW.last_master AND
			OLD.last_master_start_time = NEW.last_master_start_time AND
			OLD.last_incremental = NEW.last_incremental AND
			OLD.last_incremental_start_time = NEW.last_incremental_start_time AND
			OLD.last_selective = NEW.last_selective AND
			OLD.last_selective_start_time = NEW.last_selective_start_time AND
			OLD.last_baremetal 	= NEW.last_baremetal AND
			OLD.last_baremetal_start_time = NEW.last_baremetal_start_time AND
			OLD.priority = NEW.priority AND
			OLD.options = NEW.options AND
			OLD.features = NEW.features AND
			OLD.fstatus = NEW.fstatus AND
			OLD.properties = NEW.properties AND
			OLD.bm_media_test_date = NEW.bm_media_test_date AND
			OLD.version = NEW.version) THEN
				RETURN NULL;
			-- ignore changes to some columns. 
			ELSIF (OLD.last_master	!= NEW.last_master OR
			OLD.last_master_start_time != NEW.last_master_start_time OR
			OLD.last_incremental != NEW.last_incremental OR
			OLD.last_incremental_start_time != NEW.last_incremental_start_time OR
			OLD.last_selective != NEW.last_selective OR
			OLD.last_selective_start_time != NEW.last_selective_start_time OR
			OLD.last_baremetal 	!= NEW.last_baremetal OR
			OLD.last_baremetal_start_time != NEW.last_baremetal_start_time) THEN
				RETURN NULL;
			END IF;
		ELSIF ((TG_RELNAME = 'devices') OR (TG_RELNAME = 'bp.devices')) THEN
			IF (OLD.device_no = NEW.device_no AND
			OLD.dev_name = NEW.dev_name AND
			OLD.device_type = NEW.device_type AND
			OLD.node_no = NEW.node_no AND
			OLD.capacity = NEW.capacity AND
			OLD.blocksize = NEW.blocksize AND
			OLD.cur_fileset = NEW.cur_fileset AND
			OLD.comment = NEW.comment AND
			OLD.status = NEW.status AND
			OLD.cur_k_position = NEW.cur_k_position AND
			OLD.time_stamp = NEW.time_stamp AND
			OLD.scsi_blksize = NEW.scsi_blksize AND
			OLD.is_scsi = NEW.is_scsi AND
			OLD.dev_rw_name = NEW.dev_rw_name AND
			OLD.dev_nr_name = NEW.dev_nr_name AND
			OLD.dev_cmd_name = NEW.dev_cmd_name AND
			OLD.pid = NEW.pid AND
			OLD.same_as_devices = NEW.same_as_devices AND
			OLD.auto_switch = NEW.auto_switch AND
			OLD.tape_class = NEW.tape_class AND
			OLD.reservation_pid = NEW.reservation_pid AND
			OLD.scsi_caps = NEW.scsi_caps AND
			OLD.scsi_lun = NEW.scsi_lun AND
			OLD.scsi_id = NEW.scsi_id AND
			OLD.scsi_ha = NEW.scsi_ha AND
			OLD.tested = NEW.tested AND
			OLD.drive_level = NEW.drive_level AND
			OLD.fd = NEW.fd AND
			OLD.tape_no = NEW.tape_no AND
			OLD.properties = NEW.properties AND
			OLD.juke_drive_no = NEW.juke_drive_no AND
			OLD.no_label = NEW.no_label AND
			OLD.mux_maxcnt = NEW.mux_maxcnt AND
			OLD.mux_sched = NEW.mux_sched AND
			OLD.mux_running = NEW.mux_running AND
			OLD.is_online = NEW.is_online AND
			OLD.slot_no = NEW.slot_no AND
			OLD.dev_changer_name = NEW.dev_changer_name AND
			OLD.use_juke = NEW.use_juke AND
			OLD.juke_libno = NEW.juke_libno AND
			OLD.comment_short = NEW.comment_short AND
			OLD.total_write_usages = NEW.total_write_usages AND
			OLD.total_write_time = NEW.total_write_time AND
			OLD.backup_no = NEW.backup_no AND
			OLD.juke_slot_info = NEW.juke_slot_info AND
			OLD.juke_node_no = NEW.juke_node_no AND
			OLD.is_deleted = NEW.is_deleted AND
			OLD.dedup = NEW.dedup AND
			OLD.storage_id = NEW.storage_id) THEN
				RETURN NULL;
			ELSIF (OLD.node_no != NEW.node_no OR 
				OLD.cur_fileset != NEW.cur_fileset OR
				OLD.cur_k_position != NEW.cur_k_position OR
				OLD.reservation_pid != NEW.reservation_pid OR
				OLD.juke_drive_no != NEW.juke_drive_no OR
				OLD.mux_running != NEW.mux_running OR
				OLD.mux_sched != NEW.mux_sched OR
				OLD.backup_no != NEW.backup_no OR 
				OLD.status != NEW.status OR 
				OLD.pid != NEW.pid OR
				OLD.time_stamp != NEW.time_stamp) THEN
				RETURN NULL;
			END IF;
		ELSIF ((TG_RELNAME = 'lcs') OR (TG_RELNAME = 'bp.lcs')) THEN
			IF (OLD.lcs_no = NEW.lcs_no AND
				OLD.is_online = NEW.is_online AND
				OLD.start_time = NEW.start_time AND
				OLD.sched_name = NEW.sched_name AND
				OLD.device_no = NEW.device_no AND
				OLD.status = NEW.status AND
				OLD.last_start_time = NEW.last_start_time AND
				OLD.last_end_time = NEW.last_end_time AND
				OLD.total_netpaths = NEW.total_netpaths AND
				OLD.total_megs = NEW.total_megs AND
				OLD.job_no = NEW.job_no AND
				OLD.failure_report_is_online = NEW.failure_report_is_online AND
				OLD.failure_report_notification_type = NEW.failure_report_notification_type AND
				OLD.backup_summary_is_online = NEW.backup_summary_is_online AND
				OLD.backup_summary_notification_type = NEW.backup_summary_notification_type AND
				OLD.weekly_summary_is_online = NEW.weekly_summary_is_online AND
				OLD.weekly_summary_notification_type = NEW.weekly_summary_notification_type AND
				OLD.cmd_before = NEW.cmd_before AND
				OLD.cmd_after = NEW.cmd_after AND
				OLD.server_name = NEW.server_name AND
				OLD.last_message = NEW.last_message AND
				OLD.working_element = NEW.working_element AND
				OLD.is_recurring = NEW.is_recurring) THEN
				RETURN NULL;
			ELSIF (OLD.last_start_time != NEW.last_start_time OR
					OLD.last_end_time != NEW.last_end_time OR
					OLD.total_netpaths != NEW.total_netpaths OR
					OLD.total_megs != NEW.total_megs OR
					OLD.job_no != NEW.job_no OR
					OLD.last_message != NEW.last_message OR
					OLD.status != NEW.status OR
					OLD.working_element != NEW.working_element) THEN
				RETURN NULL;
			END IF;
		ELSIF ((TG_RELNAME = 'lcs_client') OR (TG_RELNAME = 'bp.lcs_client')) THEN
			IF (OLD.status != NEW.status OR
					OLD.last_job != NEW.last_job OR
					OLD.last_job_status != NEW.last_job_status OR
					OLD.last_backup != NEW.last_backup OR
					OLD.last_verify_job != NEW.last_verify_job OR
					OLD.last_verify_status != NEW.last_verify_status OR
					OLD.last_verify_backup != NEW.last_verify_backup OR
					OLD.last_message != NEW.last_message OR
					OLD.last_queued_time != NEW.last_queued_time) THEN
				RETURN NULL;
			END IF;
		ELSIF ((TG_RELNAME = 'systems') OR (TG_RELNAME = 'bp.systems')) THEN
			IF (OLD.check_timestamp != NEW.check_timestamp OR
				OLD.is_active != NEW.is_active) THEN
				RETURN NULL;
			END IF;
		END IF;  
	END IF;		
	-- Try update first, usually entry for table name will exist --
	UPDATE bp.modification_time SET last_changed=extract(epoch FROM now()) WHERE table_name=TG_RELNAME;
		RETURN NULL;
	IF NOT FOUND THEN
		INSERT INTO bp.modification_time(table_name) values (TG_RELNAME, extract(epoch FROM now()));
			RETURN NEW;
		END IF;
	END
	$modification_tag$ LANGUAGE 'plpgsql';

	-- Ends nasty function ------------------------------------------------------
	-- For tables that need to be part of appliance agent backup, 
	-- add trigger to track modified time --

	CREATE OR REPLACE FUNCTION bp.add_modification_triggers()
		RETURNS BOOLEAN AS 
		$add_trigger$
		DECLARE 
			table_list VARCHAR[];
			qstr VARCHAR;
		BEGIN
			-- Core tables -- 
			table_list[1] := 'storage';
			table_list[2] := 'internal';
			table_list[3] := 'nas';
			table_list[4] := 'iscsi';
			table_list[5] := 'nodes';
			table_list[6] := 'node_volumes';
			table_list[7] := 'devices';
			table_list[8] := 'tapes';
			-- Application_tables -- 
			table_list[9] := 'application_lookup';
			table_list[10] := 'node_applications';
			table_list[11] := 'application_instances';
			table_list[12] := 'application_properties';
			-- lists --
			table_list[13] := 'selection_lists'; 
			table_list[14] := 'selection_list_filenames';
			table_list[15] := 'option_lists';
			-- vmware --
			table_list[16] := 'vmware_vcenter_servers';
			table_list[17] := 'vmware_esx_servers';
			table_list[18] := 'vmware_connections';
			table_list[19] := 'vmware_vms';
			table_list[20] := 'vmware_vcenter_assoc';
			-- Scheduling --
			table_list[21] := 'lcs'; 
			table_list[22] := 'lcs_client';
			table_list[23] := 'calendars';
			table_list[24] := 'schedules';
			table_list[25] := 'schedule_client_assoc';
			table_list[26] := 'schedule_lcs';
			table_list[27] := 'strategies';
			table_list[28] := 'events';
			table_list[29] := 'byday';
			table_list[30] := 'byhour';
			-- user settings --
			table_list[31] := 'retention_limits';
			table_list[32] := 'customers';
			table_list[33] := 'locations';
			table_list[34] := 'systems';
			table_list[35] := 'users';
			table_list[36] := 'superusers';
			table_list[37] := 'customer_priv';
			table_list[38] := 'location_priv';
			table_list[39] := 'system_priv';
			table_list[40] := 'managers';
			table_list[41] := 'state';
			-- Archiving -- 
			table_list[42] := 'archive_instances';
			table_list[43] := 'archive_profiles';
			table_list[44] := 'archive_profile_backtypes';
			table_list[45] := 'archive_profile_clients';
			table_list[46] := 'archive_profile_instances';
			table_list[47] := 'archive_profile_localdirs';
			table_list[48] := 'archive_schedule_profiles';
			table_list[49] := 'archive_profile_slots';
			table_list[50] := 'archive_devices';
			table_list[51] := 'archive_device_settings';
			table_list[52] := 'archive_tapes';
			-- More core --
			table_list[53] := 'client_credentials';
			table_list[54] := 'ndmp_nas';
			-- STOP: Do not add a new table entry here. Instead, add the 
			-- table creation block _after_ the bp.modification_time block
			-- and add appropriate triggers there. See gfs_policy creation 
			-- block for an example. --		
			-- Ends table list for this revision -- 
			-- When adding new tables in the future, update 'replication' flag,
			-- and add a trigger to track modification time.
			-- Adding two different triggers because we don't want the nasty procedure being
			-- called for every row during an insert/delete. All we care is that there was
			-- and insert/delete, and not the number of rows. Updates are different though.
			FOR i IN 1..array_upper(table_list,1) LOOP
				qstr := 'CREATE TRIGGER update_modification_time_for_update_trig AFTER' ||
				' UPDATE ON ' || quote_ident(table_list[i]) || 
				' FOR EACH ROW EXECUTE PROCEDURE update_modification_time()';
				EXECUTE qstr;
				qstr := 'CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR' ||
				' DELETE ON ' || quote_ident(table_list[i]) || 
				' FOR EACH STATEMENT EXECUTE PROCEDURE update_modification_time()';
				EXECUTE qstr;

				-- Also add the above tables to modification_time table
				-- so that appliance agent knows to pick them up during backup
				qstr := 'INSERT INTO bp.modification_time(table_name) VALUES('||
						quote_literal(table_list[i]) || ')';
				EXECUTE qstr;
			END LOOP;
		RETURN TRUE;
	END
	$add_trigger$ LANGUAGE 'plpgsql' VOLATILE;
	
	 SELECT INTO mods bp.add_modification_triggers();
	 DROP FUNCTION bp.add_modification_triggers();
END IF; -- Ends creation of modification_time 


-- The xenserver tables are for the xenserver utility of the UEB
-- Begin xenserver tables
IF NOT table_exists('bp', 'xen_servers') THEN
	CREATE OR REPLACE FUNCTION bp.del_xen_server_disks()
	RETURNS TRIGGER AS
	$del_xen$
	BEGIN
		DELETE FROM bp.xen_vm_disks WHERE EXISTS
			(SELECT 1 FROM bp.application_instances WHERE
				OLD.xen_uuid = application_instances.key1 AND
				application_instances.instance_id = xen_vm_disks.instance_id);
		RETURN OLD;
	END $del_xen$
	LANGUAGE 'plpgsql' VOLATILE;
	CREATE TABLE bp.xen_servers (
		xen_uuid VARCHAR NOT NULL,
		address VARCHAR NOT NULL, 
		current BOOLEAN NOT NULL DEFAULT TRUE,
		is_pool_master BOOLEAN NOT NULL DEFAULT FALSE,
		version VARCHAR NOT NULL,
		num_cpu_sockets INTEGER NOT NULL DEFAULT 1,
		node_no INTEGER REFERENCES nodes(node_no) ON DELETE CASCADE,
		xen_server_name VARCHAR NOT NULL,
		PRIMARY KEY (xen_uuid));
	INSERT INTO bp.modification_time(table_name) VALUES ('xen_servers');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			xen_servers FOR EACH ROW EXECUTE PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON xen_servers FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER del_from_xen_servers_to_xen_vm_disks BEFORE DELETE ON
			xen_servers FOR EACH ROW EXECUTE
			PROCEDURE del_xen_server_disks();
END IF;

IF NOT table_exists('bp', 'xen_vms') THEN
	CREATE TABLE bp.xen_vms (
		vm_uuid VARCHAR NOT NULL,
		xen_uuid VARCHAR NOT NULL REFERENCES bp.xen_servers(xen_uuid) ON DELETE CASCADE,
		name VARCHAR NOT NULL, 
		instance_id INTEGER REFERENCES bp.application_instances(instance_id) NOT NULL,
		is_template BOOLEAN NOT NULL DEFAULT FALSE,
		PRIMARY KEY (vm_uuid));
	INSERT INTO bp.modification_time(table_name) VALUES ('xen_vms');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			xen_vms FOR EACH ROW EXECUTE PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON xen_vms FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
END IF;

IF NOT table_exists('bp', 'xen_vm_disks') THEN
	CREATE TABLE bp.xen_vm_disks (
		disk_uuid VARCHAR NOT NULL,
		instance_id INTEGER REFERENCES bp.application_instances(instance_id) NOT NULL,
		is_local BOOLEAN NOT NULL DEFAULT FALSE,
		is_excluded BOOLEAN NOT NULL DEFAULT FALSE,
		user_device INTEGER NOT NULL,
		name VARCHAR NOT NULL,
		gb_size INTEGER NOT NULL,
		current BOOLEAN NOT NULL DEFAULT TRUE,
		PRIMARY KEY (disk_uuid));
	INSERT INTO bp.modification_time(table_name) VALUES ('xen_vm_disks');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			xen_vm_disks FOR EACH ROW EXECUTE PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON xen_vm_disks FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
END IF;
-- End xenserver tables

-- The table gfs_policy saves system wide retention policy information -- 
IF NOT table_exists('bp', 'gfs_policy') THEN
	CREATE TABLE gfs_policy (
		policy_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		name VARCHAR(40) UNIQUE NOT NULL,
		description VARCHAR(1024) NOT NULL,
		days SMALLINT,
		weekly SMALLINT,
		monthly SMALLINT,
		yearly SMALLINT,  
		is_default BOOLEAN NOT NULL DEFAULT FALSE);
	INSERT INTO bp.modification_time(table_name) VALUES ('gfs_policy');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			gfs_policy FOR EACH ROW EXECUTE PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON gfs_policy FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
	 	
END IF;

-- This table stores associations between defined policies and instances --
IF NOT table_exists('bp', 'gfs_policy_association') THEN
	CREATE TABLE gfs_policy_association (
		instance_id INTEGER PRIMARY KEY REFERENCES application_instances (instance_id) ON DELETE CASCADE,
		policy_id INTEGER REFERENCES gfs_policy (policy_id) ON DELETE CASCADE NOT NULL
		);
	INSERT INTO bp.modification_time(table_name) VALUES ('gfs_policy_association');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			gfs_policy_association FOR EACH ROW EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON gfs_policy_association FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
END IF;

-- Tables for replication configuration
IF NOT table_exists('bp', 'source_replication_config') THEN
	CREATE TABLE source_replication_config (
		target_id SERIAL PRIMARY KEY,        -- identifier for the target (key)
		url_name VARCHAR UNIQUE,      -- target hostname or FQDN for https URLs
		target VARCHAR UNIQUE,               -- hostname of target (vcd SyncTo)
		target_type VARCHAR NOT NULL,     -- Unitrends_cloud, appliance, or MSP
		target_version VARCHAR DEFAULT 'Unknown', -- rpm install version of tgt
		status VARCHAR NOT NULL,                 -- pending, accepted, rejected
		message VARCHAR,      -- reason for rejection, startup of OpenVPN, etc.
		created BIGINT NOT NULL,           -- (epoch) when the request was sent
		updated BIGINT,                 -- (epoch) when the request was updated
		last_poll BIGINT,         -- (epoch) time of last status poll to target
		is_replication_suspended BOOLEAN DEFAULT FALSE, --did target suspend us
		suspend_toggled BIGINT DEFAULT 0  -- (epoch) when suspend state changed 
	);
	INSERT INTO bp.modification_time (table_name)
			VALUES ('source_replication_config');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			source_replication_config FOR EACH ROW EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON source_replication_config FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
END IF;

IF NOT table_exists('bp', 'target_replication_config') THEN
	CREATE TABLE target_replication_config (
		request_id SERIAL PRIMARY KEY,      -- identifier for the request (key)
		hostname VARCHAR NOT NULL, -- source's hostname, not necessarily unique
		identity VARCHAR UNIQUE NOT NULL,                 -- uuid of the source
		asset_tag VARCHAR UNIQUE NOT NULL,         -- asset tag from the source
		system_id INTEGER, -- Not a FK,can be NULL,IDs a current managed system
		status VARCHAR NOT NULL,                 -- pending, accepted, rejected
		message VARCHAR,              -- reason for rejection, or other message
		created BIGINT NOT NULL,            -- (epoch) when the request arrived
		updated BIGINT,                 -- (epoch) last time status was updated
		csr VARCHAR NOT NULL                     -- certificate signing request
	);
	INSERT INTO bp.modification_time (table_name)
			VALUES ('target_replication_config');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			target_replication_config FOR EACH ROW EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON target_replication_config FOR EACH STATEMENT EXECUTE 
			PROCEDURE update_modification_time();
END IF;

IF NOT table_exists('bp', 'replication_authcode_ip_port') THEN
	CREATE TABLE replication_authcode_ip_port (
		public_ip VARCHAR PRIMARY KEY,
		https_port INTEGER NOT NULL,
		url_name VARCHAR NOT NULL           -- match apache SSL cert commonName
	);
END IF;

IF NOT table_exists('bp', 'replication_authcodes') THEN
	CREATE TABLE replication_authcodes (
		asset_tag VARCHAR PRIMARY KEY,
		authcode VARCHAR UNIQUE NOT NULL,
		storage_id INTEGER REFERENCES storage(storage_id) ON DELETE SET NULL
	);
END IF;
-- End: tables for replication configuration

-- Replication joborders
IF NOT table_exists('bp', 'replication_joborders') THEN
	CREATE TABLE replication_joborders (
		replication_joborder_id SERIAL PRIMARY KEY UNIQUE NOT NULL,
		name VARCHAR UNIQUE NOT NULL,
		description VARCHAR NOT NULL,
		target_id INTEGER REFERENCES source_replication_config(target_id)
			ON DELETE CASCADE NOT NULL,
		enabled BOOLEAN NOT NULL DEFAULT TRUE,
		date_created BIGINT NOT NULL
	);
	INSERT INTO bp.modification_time (table_name)
			VALUES ('replication_joborders');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			replication_joborders FOR EACH ROW EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON replication_joborders FOR EACH STATEMENT
			EXECUTE PROCEDURE update_modification_time();
END IF;

IF NOT table_exists('bp', 'replication_joborders_instance_assoc') THEN
	CREATE TABLE replication_joborders_instance_assoc (
		instance_id INTEGER REFERENCES application_instances (instance_id)
			ON DELETE CASCADE UNIQUE NOT NULL,
		replication_joborder_id INTEGER
			REFERENCES replication_joborders (replication_joborder_id)
			ON DELETE CASCADE NOT NULL
	);
	INSERT INTO bp.modification_time (table_name)
			VALUES ('replication_joborders_instance_assoc');
	CREATE TRIGGER update_modification_time_for_update_trig AFTER UPDATE ON 
			replication_joborders_instance_assoc FOR EACH ROW EXECUTE 
			PROCEDURE update_modification_time();
	CREATE TRIGGER update_modification_time_for_insert_trig AFTER INSERT OR 
			DELETE ON replication_joborders_instance_assoc FOR EACH STATEMENT
			EXECUTE PROCEDURE update_modification_time();
END IF;
-- End: replication joborders


RETURN res;
END;
$BODY$

LANGUAGE 'plpgsql' VOLATILE;

SELECT create_bp();
DROP FUNCTION create_bp();

SET search_path TO statussync,public;

CREATE OR REPLACE FUNCTION create_statussync()
RETURNS INT4 AS
$BODY$DECLARE
	res INT4;
BEGIN
	res:=0;

-- begin statussync tables
IF NOT table_exists('statussync', 'sync_metadata') THEN
	CREATE TABLE sync_metadata (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		last_sync_remotetime BIGINT,
		last_sync_localtime BIGINT,
		PRIMARY KEY (system_id));
END IF;
	
IF NOT table_exists('statussync', 'sync_alerts') THEN
	CREATE TABLE sync_alerts (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		alert_id INTEGER,
		source_id INTEGER,
		source_name VARCHAR,
		severity INTEGER,
		alert_text TEXT,
		target TEXT,
		created BIGINT,
		updated BIGINT,
		closed BOOLEAN,
		PRIMARY KEY (system_id, alert_id));
END IF;
	
IF NOT table_exists('statussync', 'sync_nodes') THEN
	CREATE TABLE sync_nodes (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		node_no INTEGER,
		node_name VARCHAR,
		type INTEGER,
		os INTEGER,
		status INTEGER,
		priority INTEGER,
		properties INTEGER,
		PRIMARY KEY (system_id, node_no));
END IF;
	
IF NOT table_exists('statussync', 'sync_backups') THEN
	CREATE TABLE sync_backups (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		backup_no INTEGER,
		node_no INTEGER,
		start_time BIGINT,
		elapsed_time BIGINT,
		job_no INTEGER,
		total_netpaths INTEGER,
		total_megs INTEGER,
		status INTEGER,
		exit_code INTEGER,
		type INTEGER,
		properties INTEGER,
		PRIMARY KEY (system_id, backup_no),
		FOREIGN KEY (system_id, node_no) REFERENCES sync_nodes (system_id, node_no) ON DELETE CASCADE);
END IF;
	
IF NOT table_exists('statussync', 'sync_sds') THEN
	CREATE TABLE sync_sds (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		backup_no BIGINT NOT NULL,
		sync_type INTEGER NOT NULL,
		presync_seconds INTEGER,
		sync_seconds INTEGER,
		postsync_seconds INTEGER,
		done_epoch BIGINT NOT NULL,
		bytes_syncd BIGINT,
		message TEXT NOT NULL);
END IF;

IF NOT table_exists('statussync', 'sync_sql_backups') THEN
	CREATE TABLE sync_sql_backups (
		system_id INTEGER REFERENCES bp.systems (system_id) ON DELETE CASCADE,
		sql_ref BIGINT,
		node_no INTEGER,
		database_name VARCHAR,
		server_instance_name VARCHAR,
		group_file_order BIGINT,
		file_order_no BIGINT,
		sql_type INTEGER,
		PRIMARY KEY (system_id, sql_ref));
END IF;
-- end of statussync tables

RETURN res;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT create_statussync();
DROP FUNCTION create_statussync();

DROP FUNCTION public.table_exists(sch_name VARCHAR,tbl_name VARCHAR);
DROP FUNCTION public.sequence_exists(sch_name VARCHAR, seq_name VARCHAR);
