-- This script applies modifications to the notification framework.  The
-- revision and status are stored in the notification_mgmt table.  A new
-- revision can involve changes at install time, applied by this script,
-- and/or may involve migration of data in the alerts table, applied by
-- program migrate_alerts.php on the first run of alertman after an rpm
-- upgrade.  The status field with have a value of 'needs migration' if
-- migrate_alerts.php should run.
--
-- There is only one record in the notification_mgmt table, with the
-- following values:
--   revision:     the revision level that is currently installed.
--   upgrade_from: the previous level (an rpm upgrade may apply several
--                 revs); this value is transient and only used if data
--                 migration needs to be performed.  Its value is equal
--                 to the current revision if alertman has checked the
--                 status and finalized the upgrade.  Note that it might be
--                 possible (rare) that multiple upgrades occurred without
--                 an intervening run of alertman.
--   status: string, indicates if alertman should run migration, or if all
--                 changes for the revision are complete.  If migration is
--                 needed, alertman will apply any migrations between the
--                 values of "upgrade_from" and "revision" and then update
--                 the notification_mgmt table to indicate final completion.
--
-- The function is separated into blocks, one for each revision.
-- To add changes for a new revision:
--  1. Increment the value of the new_rev variable in the DECLARE block.
--  2. Add a new block with condition "current_revision < new_rev".
--  3. If the revision requires data migration, set the update_status variable
--     in the block to 'needs migration' (add a case to the migrator program
--     to accomplish the migration).  Do not set the value of update_status
--     to complete!  Only set the value if some data in the alerts table
--     needs migration (or some other dynamically generated data).
--
-- This is the basic structure of the function:
--  DECLARE
--  new_rev INTEGER := 3;     <== change this value below in the code
--
--  (...statements to get current_revision & update_status...)
--	IF (current_revision < 1)
--	THEN
--		SQL statements for rev 1
--	END IF;
--	IF (current_revision < 2)
--	THEN
--		SQL statements for rev 2
--      update_status := 'needs migration';
--	END IF;
--	IF (current_revision < new_rev)
--	THEN
--		SQL statements for new_rev
--	END IF;
--
--	UPDATE notification_mgmt SET revision = new_rev, status = update_status;
--

SET search_path TO bp,public;

CREATE OR REPLACE FUNCTION update_notifications()
RETURNS BOOLEAN AS
$BODY$

DECLARE
	NMrow notification_mgmt%ROWTYPE;
	new_rev INTEGER := 19;
	current_revision INTEGER;
	update_status VARCHAR;

BEGIN

-- There should be only one row, the ORDER BY & LIMIT are just in case...
SELECT INTO NMrow * FROM notification_mgmt ORDER BY revision DESC LIMIT 1 ;

current_revision := NMrow.revision;
update_status := NMrow.status;

IF (current_revision >= new_rev)
THEN
	RETURN TRUE;
END IF;

IF (current_revision < 1)
THEN
--  Inserts for categories table, columns are: category_id, category
	INSERT INTO categories VALUES (0, 'Internal');
	INSERT INTO categories VALUES (1, 'Platform');
	INSERT INTO categories VALUES (2, 'Storage');
	INSERT INTO categories VALUES (3, 'Core');
	INSERT INTO categories VALUES (4, 'Backup');
	INSERT INTO categories VALUES (5, 'Restore');
	INSERT INTO categories VALUES (6, 'Management');
	INSERT INTO categories VALUES (7, 'Archiving');
	INSERT INTO categories VALUES (8, 'Vaulting');

	INSERT INTO trap_types VALUES (1, 'Clients state has changed');
	INSERT INTO trap_types VALUES (2, '3ware state has changed');
	INSERT INTO trap_types VALUES (3, 'PCI state has changed');
	INSERT INTO trap_types VALUES (5, 'Process state has changed');
	INSERT INTO trap_types VALUES (6, 'Disk state has changed');
	INSERT INTO trap_types VALUES (7, 'CEP state has changed');
	INSERT INTO trap_types VALUES (8, 'DPU state has changed');
	INSERT INTO trap_types VALUES (9, 'RAID state has changed');
	INSERT INTO trap_types VALUES (10, 'Backup state has changed');
	INSERT INTO trap_types VALUES (99, 'Test Trap');

--- Initial value for non-migrated alert records
	INSERT INTO notifications VALUES(0, 0, 'Internal notice',
	'Non-migrated alert record',
	0, true, NULL, false, false, false);

--  Add new column to alerts; alertman will migrate/assign values
	ALTER TABLE alerts ADD COLUMN notification_id INTEGER REFERENCES
		notifications(notification_id) DEFAULT 0;

--- Inserts for notifications table, columns are:
---   notification_id category_id short_description primary_text
---   severity state action alert trap email
--- alert_source 3ware
	INSERT INTO notifications VALUES(1, 0, 'Internal notice',
	'Resolve/Close alerts for 3ware alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(2, 2, '3ware state has changed',
	'3ware %s unit status changed %s',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(3, 2, '3ware state has changed',
	'3ware %s drive status changed %s',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(4, 2, '3ware state has changed',
	'3ware %s unit status changed %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(5, 2, '3ware state has changed',
	'3ware %s drive status changed %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(6, 2, '3ware state has changed',
	'3ware %s unit status changed %s',
	3, true, NULL, true, true, false);

	INSERT INTO notifications VALUES(7, 2, '3ware state has changed',
	'3ware %s drive status changed %s',
	3, true, NULL, true, true, false);

--- alert_source Clients
	INSERT INTO notifications VALUES(8, 0, 'Internal notice',
	'Resolve/Close alerts for Clients alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(9, 4, 'Client state has changed',
	'%s: client is no longer listening to connections. %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(10, 4, 'Client state has changed',
	'%s: client is now listening to connections. %s',
	3, true, NULL, true, true, false);

--- alert_source Disk
	INSERT INTO notifications VALUES(11, 0, 'Internal notice',
	'Resolve/Close alerts for Disk alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(12, 2, 'Disk space usage warning',
	'Disk space usage threshold exceeded. Disk %s is %s full (%s MB free).',
	2, false, NULL, true, true, false);

--- alert_source DPU
	INSERT INTO notifications VALUES(13, 0, 'Internal notice',
	'Resolve/Close alerts for DPU alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(14, 6, 'DPU state has changed',
	'%s: cannot connect to DPU database. %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(15, 6, 'DPU state has changed',
	'%s: DPU database is now responding. %s',
	3, true, NULL, true, true, false);

--- alert_source PCI
	INSERT INTO notifications VALUES(16, 0, 'Internal notice',
	'Resolve/Close alerts for PCI alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(17, 1, 'PCI state has changed',
	'PCI device has been disconnected: %s',
	2, true, NULL, true, true, false);

	INSERT INTO notifications VALUES(18, 1, 'PCI state has changed',
	'PCI device has been installed: %s',
	3, true, NULL, true, true, false);

--- alert_source Process
	INSERT INTO notifications VALUES(19, 0, 'Internal notice',
	'Resolve/Close alerts for Process alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(20, 4, 'License expiration',
	'License: DPU license has expired.',
	1, false, NULL, true, true, true);

	INSERT INTO notifications VALUES(21, 3, 'CryptoDaemon process state has changed',
	'Process: cryptoDaemon is required but not running.',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(22, 3, 'CryptoDaemon process state has changed',
	'Process: cryptoDaemon is required and running but has no master key.',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(23, 4, 'Licensed capacity exceeded',
	'License Grace Period: The licensed capacity of this DPU has been exceeded.',
	1, false,
	'Please see the system e-mail for urgent upgrade instructions.', true, true, true);

	INSERT INTO notifications VALUES(24, 4, 'Licensed capacity exceeded',
	'License Failure Notification: This DPU cannot perform backups because your data has exceeded the licensed capacity.',
	1, false,
	'Please see the system e-mail for urgent upgrade instructions.', true, true, true);

	INSERT INTO notifications VALUES(25, 6, 'New Rapid Recovery version available',
	'Update: Unitrends Rapid Recovery version %s is available.',
	1, false,
	'To install, go to the Rapid Recovery Console->Configure->Updates.', true, true, false);

	INSERT INTO notifications VALUES(26, 3, 'Tasker process state has changed',
	'Process: Tasker has been stopped.',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(27, 4, 'Licensed capacity warning',
	'License Alert: This DPU is at 70%% of its licensed capacity.',
	2, false,
	'Please see the system e-mail for upgrade instructions.', true, true, true);

	INSERT INTO notifications VALUES(28, 4, 'Licensed capacity warning',
	'License Warning (Yellow Alert): This DPU is at 80%% of its licensed capacity.',
	2, false,
	'Please see the system e-mail for urgent upgrade instructions.', true, true, true);

	INSERT INTO notifications VALUES(29, 4, 'Licensed capacity warning',
	'Urgent License Warning (Red Alert): This DPU is at 90%% of its licensed capacity.  Backup failures are imminent.',
	2, false,
	'Please see the system e-mail for urgent upgrade instructions.', true, true, true);

	INSERT INTO notifications VALUES(30, 4, 'Retention tool warning',
	'Retention: The DPU retention tool was unable to calculate the retention period.',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(31, 4, 'Retention tool warning',
	'Retention: The DPU retention tool has not been initialized.',
	2, false, NULL, true, true, true);

	INSERT INTO notifications VALUES(32, 4, 'Retention tool warning',
	'Retention: The DPU retention tool initialization grace period has expired.',
	2, false, NULL, true, true, true);

	INSERT INTO notifications VALUES(33, 1, 'Reboot required',
	'A software update requires the DPU to be rebooted.',
	2, false,
	'Go to the Rapid Recovery Console->Tools->Shutdown.', true, true, false);

	INSERT INTO notifications VALUES(34, 1, 'Driver installation',
	'unitrends-drivers has not been installed.  Will retry in 24 hours.',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(35, 3, 'Migration process state has changed',
	'Migration: long pathname migration %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(36, 4, 'Data Assurance notice',
	'Data Assurance: Your Data Assurance %s.',
	2, true,
	'To renew it, please call the Unitrends Support Center or your Authorized Partner.', true, true, false);

	INSERT INTO notifications VALUES(37, 4, 'Data Assurance notice',
	'Data Assurance: Your Data Assurance expires on %s.',
	3, false,
	'To renew it, please call the Unitrends Support Center or your Authorized Partner.', true, true, false);

	INSERT INTO notifications VALUES(38, 3, 'Tasker process state has changed',
	'Process: Tasker has been started.',
	3, true, NULL, true, true, false);

	INSERT INTO notifications VALUES(39, 3, 'CryptoDaemon process state has changed',
	'Process: cryptoDaemon has been started.',
	3, true, NULL, true, true, false);

--- alert_source RAID
	INSERT INTO notifications VALUES(40, 0, 'Internal notice',
	'Resolve/Close alerts for RAID alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(41, 2, 'RAID state has changed',
	'RAID drive status changed to offline. %s',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(42, 2, 'RAID state has changed',
	'RAID %s: Drive status %s',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(43, 2, 'RAID state has changed',
	'RAID %s: Array status %s ',
	1, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(44, 2, 'RAID state has changed',
	'RAID %s: Drive status %s',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(45, 2, 'RAID state has changed',
	'RAID %s: Array status %s ',
	2, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(46, 2, 'RAID state has changed',
	'RAID %s: Drive status %s',
	3, false, NULL, true, true, false);

	INSERT INTO notifications VALUES(47, 2, 'RAID state has changed',
	'RAID %s: Drive status %s',
	3, true , NULL, true, true, false);

	INSERT INTO notifications VALUES(48, 2, 'RAID state has changed',
	'RAID %s: Array status %s ',
	3, true, NULL, true, true, false);

--- alert_source CEP
	INSERT INTO notifications VALUES(49, 0, 'Internal notice',
	'Resolve/Close alerts for CEP alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(50, 4, 'CEP state has changed',
	'CEP workspace %s has errors today.',
	1, false,
	'Please check Configure->Exchange->View Error Log.', true, true, false);

	INSERT INTO notifications VALUES(51, 4, 'CEP state has changed',
	'Global Exchange log has errors today.',
	1, false,
	'Please check Configure->Exchange->View Global Error Log.', true, true, false);

--- alert_source Backup
	-- we don't really need the Resolve because this source only sends traps
	INSERT INTO notifications VALUES(52, 0, 'Internal notice',
	'Resolve/Close alerts for Backup alert source',
	0, true, NULL, true, false, false);

	INSERT INTO notifications VALUES(53, 4, 'Backup failure',
	'Scheduled backup did not run.  %s',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(54, 4, 'Backup failure',
	'Master backup for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(55, 4, 'Backup failure',
	'Incremental backup for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(56, 4, 'Backup failure',
	'Selective backup for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(57, 4, 'Backup failure',
	'SQL Server backup for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(58, 5, 'Restore failure',
	'Restore for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(59, 4, 'Verification failure',
	'Backup verify for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(60, 4, 'Backup failure',
	'BareMetal backup for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(61, 5, 'Restore failure',
	'BareMetal restore for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(62, 4, 'Backup failure',
	'Backup operation for client %s failed.',
	2, false, NULL, false, true, false);

	INSERT INTO notifications VALUES(63, 6, 'Test Trap',
	'Verify that SNMP configuration is working',
	3, true, NULL, false, true, false);

--  Inserts for notify_alerts table, columns are: notification_id, source_id
--- alert_source 3ware
	INSERT INTO notify_alerts VALUES(1, 2);
	INSERT INTO notify_alerts VALUES(2, 2);
	INSERT INTO notify_alerts VALUES(3, 2);
	INSERT INTO notify_alerts VALUES(4, 2);
	INSERT INTO notify_alerts VALUES(5, 2);
	INSERT INTO notify_alerts VALUES(6, 2);
	INSERT INTO notify_alerts VALUES(7, 2);
--- alert_source Clients
	INSERT INTO notify_alerts VALUES(8, 1);
	INSERT INTO notify_alerts VALUES(9, 1);
	INSERT INTO notify_alerts VALUES(10, 1);
--- alert_source Disk
	INSERT INTO notify_alerts VALUES(11, 6);
	INSERT INTO notify_alerts VALUES(12, 6);
--- alert_source DPU
	INSERT INTO notify_alerts VALUES(13, 8);
	INSERT INTO notify_alerts VALUES(14, 8);
	INSERT INTO notify_alerts VALUES(15, 8);
--- alert_source PCI
	INSERT INTO notify_alerts VALUES(16, 3);
	INSERT INTO notify_alerts VALUES(17, 3);
	INSERT INTO notify_alerts VALUES(18, 3);
--- alert_source Process
	INSERT INTO notify_alerts VALUES(19, 5);
	INSERT INTO notify_alerts VALUES(20, 5);
	INSERT INTO notify_alerts VALUES(21, 5);
	INSERT INTO notify_alerts VALUES(22, 5);
	INSERT INTO notify_alerts VALUES(23, 5);
	INSERT INTO notify_alerts VALUES(24, 5);
	INSERT INTO notify_alerts VALUES(25, 5);
	INSERT INTO notify_alerts VALUES(26, 5);
	INSERT INTO notify_alerts VALUES(27, 5);
	INSERT INTO notify_alerts VALUES(28, 5);
	INSERT INTO notify_alerts VALUES(29, 5);
	INSERT INTO notify_alerts VALUES(30, 5);
	INSERT INTO notify_alerts VALUES(31, 5);
	INSERT INTO notify_alerts VALUES(32, 5);
	INSERT INTO notify_alerts VALUES(33, 5);
	INSERT INTO notify_alerts VALUES(34, 5);
	INSERT INTO notify_alerts VALUES(35, 5);
	INSERT INTO notify_alerts VALUES(36, 5);
	INSERT INTO notify_alerts VALUES(37, 5);
	INSERT INTO notify_alerts VALUES(38, 5);
	INSERT INTO notify_alerts VALUES(39, 5);
--- alert_source RAID
	INSERT INTO notify_alerts VALUES(40, 9);
	INSERT INTO notify_alerts VALUES(41, 9);
	INSERT INTO notify_alerts VALUES(42, 9);
	INSERT INTO notify_alerts VALUES(43, 9);
	INSERT INTO notify_alerts VALUES(44, 9);
	INSERT INTO notify_alerts VALUES(45, 9);
	INSERT INTO notify_alerts VALUES(46, 9);
	INSERT INTO notify_alerts VALUES(47, 9);
	INSERT INTO notify_alerts VALUES(48, 9);
--- alert_source CEP
	INSERT INTO notify_alerts VALUES(49, 7);
	INSERT INTO notify_alerts VALUES(50, 7);
	INSERT INTO notify_alerts VALUES(51, 7);

--  Inserts for notify_traps table, columns are: notification_id, trap_type, oid
--- alert_source 3ware
	INSERT INTO notify_traps VALUES(2, 2, '100.0.2');
	INSERT INTO notify_traps VALUES(3, 2, '100.0.2');
	INSERT INTO notify_traps VALUES(4, 2, '100.0.2');
	INSERT INTO notify_traps VALUES(5, 2, '100.0.2');
	INSERT INTO notify_traps VALUES(6, 2, '100.0.2');
	INSERT INTO notify_traps VALUES(7, 2, '100.0.2');
--- alert_source Clients
	INSERT INTO notify_traps VALUES(9, 1, '100.0.1');
	INSERT INTO notify_traps VALUES(10, 1, '100.0.1');
--- alert_source Disk
	INSERT INTO notify_traps VALUES(12, 6, '100.0.6');
--- alert_source DPU
	INSERT INTO notify_traps VALUES(14, 8, '100.0.8');
	INSERT INTO notify_traps VALUES(15, 8, '100.0.8');
--- alert_source PCI
	INSERT INTO notify_traps VALUES(17, 3, '100.0.3');
	INSERT INTO notify_traps VALUES(18, 3, '100.0.3');
--- alert_source Process
	INSERT INTO notify_traps VALUES(20, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(21, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(22, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(23, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(24, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(25, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(26, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(27, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(28, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(29, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(30, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(31, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(32, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(33, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(34, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(35, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(36, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(37, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(38, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(39, 5, '100.0.5');
--- alert_source RAID
	INSERT INTO notify_traps VALUES(41, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(42, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(43, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(44, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(45, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(46, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(47, 9, '100.0.9');
	INSERT INTO notify_traps VALUES(48, 9, '100.0.9');
--- alert_source CEP
	INSERT INTO notify_traps VALUES(50, 7, '100.0.7');
	INSERT INTO notify_traps VALUES(51, 7, '100.0.7');
--- alert_source Backup
	INSERT INTO notify_traps VALUES(53, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(54, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(55, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(56, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(57, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(58, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(59, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(60, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(61, 10, '100.0.10');
	INSERT INTO notify_traps VALUES(62, 10, '100.0.10');
--  Test trap (no alert source)
	INSERT INTO notify_traps VALUES(63, 99, '100.0.99');

--  Inserts for notify_emails table, columns are:
--  notification_id, to_section, to_keyword, from_name, subject, body
	INSERT INTO notify_emails VALUES(20, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'License has expired.',
	'Your Unitrends DPU license has expired.  Please renew your license.');

	INSERT INTO notify_emails VALUES(23, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'FATAL: License grace period.',
	'License Grace Period: The licensed capacity of this Data Protection Unit has been exceeded. Usage above your current allowance is permitted for a grace period of 7 days (total).  Once this grace period expires, any attempt to perform backups to your Data Protection Unit will automatically be denied by the system.  Your grace period will expire in %s days.  Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to increase your licensed capacity and restart the process of protecting your data.');

	INSERT INTO notify_emails VALUES(24, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'FATAL: License has expired.',
	'License Failure Notification:  The Total System Content being protected by this Data Protection Unit has now exceeded the licensed capacity (%s GB).  The system is now unable to perform backups until you increase your licensed capacity.  This situation is critical.  Your data is no longer being protected.  Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to increase your licensed capacity and restart the process of protecting your data.');

	INSERT INTO notify_emails VALUES(27, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'WARNING: License threshold exceeded.',
	'License Alert: The licensed capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents greater than 70%% of the licensed capacity.  Many customers experience growth in data volume of as much as 100%% per year, which means you may soon need to increase your license to ensure that all your data is fully protected.  You can upgrade your license by contacting your authorized Unitrends partner or Unitrends representative.');

	INSERT INTO notify_emails VALUES(28, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'WARNING: License threshold exceeded.',
	'License Warning (Yellow Alert): The licensed capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents greater than 80%% of the licensed capacity.  Many customers experience growth in data volume of as much as 100%% per year, which means it''s becoming urgent that you increase your license to ensure that all your data is fully protected.  Your backups will not run correctly if your data volume exceeds your license.  You can upgrade your license now by contacting your authorized Unitrends partner or Unitrends representative (at 888-374-6124).');

	INSERT INTO notify_emails VALUES(29, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'WARNING: License threshold exceeded.',
	'License Warning (Red Alert): The licensed capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents greater than 90%% of the licensed capacity.  You should immediately increase your license to ensure that all your data is fully protected.  Your backups will not run correctly if your data volume exceeds your license, and this situation is imminent.  Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to avoid preventable backup failures.');

	INSERT INTO notify_emails VALUES(31, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'WARNING: License setup grace period.',
	'License Setup Grace Period Notification: The license for this Data Protection Unit has not been initialized.  You must initialize your license within 30 days of upgrade or installation.  Once this grace period expires, any attempt to perform backups to your Data Protection Unit will automatically be denied by the system.  Your grace period will expire in %s days.  Please contact your authorized Unitrends partner or Unitrends Support Center immediately at 888-374-6124 to initialize your license properly and restart the process of protecting your data.');

	INSERT INTO notify_emails VALUES(32, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'FATAL: License setup grace period has expired.',
	'License Setup Grace Period Notification: The license for this Data Protection Unit has not been initialized.  The system is now unable to perform backups until you initialize your license. This situation is critical.  Your data is no longer being protected.  Please contact your authorized Unitrends partner or Unitrends representative immediately at 888-374-6124 to initialize your license and restart the process of protecting your data.');

	update_status := 'needs migration';

END IF;


IF (current_revision < 2)
THEN
	ALTER TABLE notifications ADD COLUMN log BOOLEAN NOT NULL DEFAULT FALSE;

--- Inserts for notifications table, now the columns are:
---   notification_id category_id short_description primary_text
---   severity state action alert trap email log
--- Note: category=Storage, trap_type='Disk state has changed'
	INSERT INTO notifications VALUES(64, 2, 'Deleted backup(s)',
	'Backup(s) for client %s marked for deletion: #%s',
	3, true, NULL, false, true, false, true);
	INSERT INTO notify_traps VALUES(64, 6, '100.0.6');

END IF;


IF (current_revision < 3)
THEN
	INSERT INTO trap_types VALUES (11, 'Archiving state has changed');

--- Inserts for notifications table, columns are:
---   notification_id category_id short_description primary_text
---   severity state action alert trap email
--- Note: category='Archiving', trap_type='Archiving state has changed'
	INSERT INTO notifications VALUES(65, 7, 'DPU Archiving state has changed',
	'Archive job %s is running.',
	1, false, NULL, true, true, false);
	INSERT INTO notifications VALUES(66, 7, 'DPU Archiving state has changed',
	'Archive job %s is not running.',
	1, true, NULL, true, true, false);

	INSERT INTO notify_alerts VALUES(65, 11);
	INSERT INTO notify_alerts VALUES(66, 11);

	INSERT INTO notify_traps VALUES(65, 11, '100.0.11');
	INSERT INTO notify_traps VALUES(66, 11, '100.0.11');
END IF;

IF (current_revision < 4)
THEN
	UPDATE notify_emails
	SET body = 'Your Unitrends Appliance license has expired.  Please renew your license by calling 888-374-6124.'
	WHERE notification_id = 20;
END IF;

IF (current_revision < 5)
THEN
	UPDATE notifications SET severity = 2 WHERE notification_id = 25;
END IF;

IF (current_revision < 6)
THEN
	INSERT INTO notifications VALUES(67, 4, 'SQL Server schedule skipped',
	'SQL Server Instance %s on client %s will not perform scheduled backups for database %s because backups of type %s are not supported.',
	1, true, NULL, true, true, false);
	INSERT INTO notify_alerts VALUES(67, 10);
	INSERT INTO notify_traps VALUES(67, 10, '100.0.10');
END IF;

IF (current_revision < 7)
THEN
	-- Note: category='Archiving', trap_type='Archiving state has changed'
	-- Drive capacity warning
	INSERT INTO notifications VALUES(68, 7, 'DPU Archiving state has changed',
	'Archive media is at %s%% of capacity.', 2, true, NULL, true, true, false);
	-- Drive capacity impending failure
	INSERT INTO notifications VALUES(69, 7, 'DPU Archiving state has changed',
	'Archive media is at %s%% of capacity.', 1, true, NULL, true, true, false);
	-- Archive status email
	INSERT INTO notifications VALUES(70, 7, 'DPU Archiving state has changed',
	'%s', 2, true, NULL, false, false, true);

	INSERT INTO notify_alerts VALUES(68, 11);
	INSERT INTO notify_alerts VALUES(69, 11);

	INSERT INTO notify_traps VALUES(68, 11, '100.0.11');
	INSERT INTO notify_traps VALUES(69, 11, '100.0.11');

	INSERT INTO notify_emails VALUES(70, 'Unattended Backup Information',
	'MailTo', 'Archive Job Notifications <root>',
	'Archive job completion', '%s');
END IF;

IF (current_revision < 8)
THEN
	INSERT INTO trap_types VALUES(12, 'Version Information');
	INSERT INTO trap_types VALUES(13, 'DPU Open Alerts Status');
	INSERT INTO trap_types VALUES(14, 'Network Status has changed');
	
	-- NOTE: category=Platform, trap_type=Version Information
	-- Version Information (no alert_source)
	INSERT INTO notifications VALUES(71, 1, 'DPU Version',
	'DPU %s',
	3, true, NULL, false, true, false, false);

	INSERT INTO notifications VALUES(72, 1, 'Agent Version',
	'Node Name: %s, Version: %s, Status: %s',
	3, true, NULL, false, true, false, false);

	INSERT INTO notifications VALUES(73, 1, '3ware Information',
	'3ware %s, %s, %s',
	3, true, NULL, false, true, false, false);

	INSERT INTO notifications VALUES(74, 1, 'NIC Collisions',
	'%s: %s',
	1, false, NULL, false, true, false, false);
	
	INSERT INTO notifications VALUES(75, 1, 'NIC Dropped Packets',
	'%s: %s',
	1, false, NULL, false, true, false, false);

	INSERT INTO notifications VALUES(76, 1, 'NIC Errors',
	'%s: %s',
	1, false, NULL, false, true, false, false); 

	INSERT INTO notifications VALUES(77, 8, 'OpenVPN/Vaulting Status',
	'OpenVPN not enabled while Vaulting is enabled',
	1, false, NULL, false, true, false, false);

	INSERT INTO notify_traps VALUES(71, 12, '100.0.12');
	INSERT INTO notify_traps VALUES(72, 12, '100.0.12');
	INSERT INTO notify_traps VALUES(73, 12, '100.0.12');
	INSERT INTO notify_traps VALUES(74, 14, '100.0.14');
	INSERT INTO notify_traps VALUES(75, 14, '100.0.14');
	INSERT INTO notify_traps VALUES(76, 14, '100.0.14'); 
	INSERT INTO notify_traps VALUES(77, 14, '100.0.14');
END IF;

IF (current_revision < 9)
THEN
	INSERT INTO notifications VALUES(78, 8,
	'Vaulting operation terminated or reset',
	'%s a vaulting operation for %s''s %s.',
	3, true, NULL, false, true, false, true);

	INSERT INTO notify_traps VALUES(78, 5, '100.0.5');
END IF;

IF (current_revision < 10)
THEN

	-- NOTE: category=Core, trap_type="Process state has changed"
	---NOTE: Inserts for notifications table, columns are:
	---   notification_id, category_id, short_description, primary_text
	---   severity, state, action, alert, trap, email, log

	-- alert_source Process
	INSERT INTO notifications VALUES(79, 3, 'ArchiveDaemon process state has changed',
	'Process: archiveDaemon is not running',
	1, false, NULL, true, true, false, false);

	INSERT INTO notifications VALUES(80, 3, 'ArchiveDaemon process state has changed',
	'Process: archiveDaemon is running',
	3, true, NULL, true, true, false, false);

	INSERT INTO notify_alerts VALUES(79, 5);
	INSERT INTO notify_alerts VALUES(80, 5); 
	
	INSERT INTO notify_traps VALUES(79, 5, '100.0.5');
	INSERT INTO notify_traps VALUES(80, 5, '100.0.5');
END IF;

IF (current_revision < 11)
THEN

	INSERT INTO notifications VALUES(81, 2, 'SMART Information',
	'%s',
	3, true, NULL, false, true, false, false);

	INSERT INTO notify_traps VALUES(81, 6, '100.0.6');
END IF;

IF (current_revision < 12)
THEN
	INSERT INTO notifications VALUES(82, 3,
	'Migration process state has changed',
	'Update: Processing of existing backups to import file data into the database is %s.',
	3, false, NULL, true, false, false, false);
	INSERT INTO notify_alerts VALUES(82, 5);
END IF;

IF (current_revision < 13)
THEN
	--alert_source Disk
	INSERT INTO notifications VALUES(83, 2, 'External Storage status',
	'External Storage: %s is Online',
	3, true, NULL, true, true, false, false);

	INSERT INTO notifications VALUES(84, 2, 'External Storage status',
	'External Storage: %s is Offline',
	1, false, NULL, true, true, false, false); 

	INSERT INTO notify_alerts VALUES(83, 6);
	INSERT INTO notify_alerts VALUES(84, 6);

	INSERT INTO notify_traps VALUES(83, 6, '100.0.6');
	INSERT INTO notify_traps VALUES(84, 6, '100.0.6');
END IF;

IF (current_revision < 14)
THEN
	
	INSERT INTO notifications VALUES(85, 0, 'Internal Notice',
	'Resolve/Close alerts for Vault alert source',
	0, true, NULL, true, false, false, false);

	INSERT INTO notifications VALUES(86, 8, 'Vault Capacity Warning',
	'Vault License Alert (Red Alert): Licensed Vault capacity has been met or exceeded',
	1, false, 
	'Please see system e-mail for urgent upgrade instructions', true, true, true, false);

	INSERT INTO notifications VALUES(87, 8, 'Vault Capacity Warning',
	'Vault License Warning (Yellow Alert): Vault is at 80%% of its licensed capacity',
	2, false, 
	'Please see system e-mail for urgent upgrade instructions', true, true, true, false);

	INSERT INTO notify_alerts VALUES(85, 6);
	INSERT INTO notify_alerts VALUES(86, 6);
	INSERT INTO notify_alerts VALUES(87, 6);

	INSERT INTO notify_traps VALUES(86, 6, '100.0.6');
	INSERT INTO notify_traps VALUES(87, 6, '100.0.6');

	INSERT INTO notify_emails VALUES(86, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'Fatal: Vault License threshold exceeded.',
	'Vault License Warning (Red Alert): The licensed vault capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents greater than 100%% of the licensed capacity. Many customers experience growth in data volume of as much as 100%% per year, which means it''s becoming urgent that you increase your license to ensure that all your data is fully protected.  Your vaulting will disabled in 5 days timeand remain disabled until your license capacity exceeds your data volume.  Please contact your Vault Service Provider for further information.');	

	INSERT INTO notify_emails VALUES(87, 'Unattended Backup Information',
	'MailTo', 'License Notifications <root>', 'WARNING: Vault License threshold exceeded.',
	'Vault License Warning (Yellow Alert): The licensed vault capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents %s%% of the licensed capacity.  Many customers experience growth in data volume of as much as 100%% per year, which means it''s becoming urgent that you increase your license to ensure that all your data is fully protected.  Your vaulting will not run correctly if your data volume exceeds your vault license capacity.  Please contact your Vault Service Provider for further information.');	
END IF;

IF (current_revision < 15)
THEN
	-- NOTE: category=Platform, trap_type=BP Agent Version Information
	---NOTE: Inserts for notifications table, columns are:
	---   notification_id, category_id, short_description, primary_text
	---   severity, state, action, alert, trap, email, log

	INSERT INTO notifications VALUES(88, 1, 'BP Agent Version Information',
	'%s out of %s clients are running an outdated version of BP Agent.',
	2, false, 'Please upgrade your BP Agent.', true, true, false, false);

	INSERT INTO notify_alerts VALUES(88, 1); 

	INSERT INTO notify_traps VALUES(88, 1, '100.0.1');
END IF; 
 
IF (current_revision < 16)
THEN
	UPDATE notifications
	  SET primary_text = 'Differential backup for client %s failed.'
	WHERE notification_id = 55;
END IF; 

IF (current_revision < 17)
THEN
	-- 5.0.2 beta wording was different, make the database consistent as of 5.1
	UPDATE notifications
	  SET primary_text = 'Update: Processing of existing backups to import file data into the database is %s.', severity = 3
	WHERE notification_id = 82;
END IF; 

IF (current_revision < 18)
THEN
	INSERT INTO notifications VALUES(89, 2, '3ware controller reset Alert',
	'3ware reset AEN:cache synchronization failed, some data lost.',
	1, false, NULL, false, true, false, false);

	INSERT INTO notify_traps VALUES(89, 2, '100.0.2');
END IF; 

IF (current_revision < 19)
THEN
	UPDATE notify_emails
	  SET body = 'Vault License Warning (Red Alert): The licensed vault capacity of this Data Protection Unit is %s GB, and your Total System Content being protected is now %s GB.  This represents greater than 100%% of the licensed capacity. Many customers experience growth in data volume of as much as 100%% per year, which means it''s becoming urgent that you increase your license to ensure that all of your data is fully protected.  Please contact your Vault Service Provider for further information.'
	WHERE notification_id = 86;

	UPDATE notifications
	  SET primary_text = 'Vault License Warning (Yellow Alert): Vault has crossed 80%% of its licensed capacity'
	WHERE notification_id = 87;
END IF;

--- Add next revision block here
--	IF (current_revision < new_rev)
--	THEN
--		SQL statements for new_rev
--	END IF;


--- Finalize the notification_mgmt record
	UPDATE notification_mgmt SET revision = new_rev, status = update_status;

RETURN TRUE;
END$BODY$
LANGUAGE 'plpgsql' VOLATILE;

SELECT update_notifications();
DROP FUNCTION update_notifications();

